#!/usr/bin/python -tt
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# $Id: yumex.py,v 1.57.2.13 2005/10/31 08:03:09 timlau Exp $

import os
import sys
import gtk
import gtk.glade
import gobject
import ConfigParser
import re
import time
import gc

from types import *

from yumexClasses import *
from yumexClasses.i18n import _
from optparse import OptionParser
import locale
      

class MainApplication:

    def __init__( self ):
     
        #gc.set_debug(gc.DEBUG_OBJECTS|gc.DEBUG_UNCOLLECTABLE)
        self.VERSION = "0.42"
        self.BUILD = "09"
        self.MAIN_PATH = os.path.abspath( os.path.dirname( sys.argv[0] ) );
        self.GLADE_FILE = self.MAIN_PATH+'/yumex.glade'      
        self.PIXMAPS_PATH = '/usr/share/pixmaps/yumex'
        self.Mega = 1024*1024
        self.cmdline_options()
        self.repoEnabled = []
        self.repoDisabled = []
        self.base_row = -1
        self.updates = -1
        self.nameFilter = ""
        self.RecentState = [False,False,False,False]
        locale.setlocale( locale.LC_ALL, "" )
        gtk.glade.bindtextdomain( 'yumex', '/usr/share/locale' )
        gtk.glade.textdomain ( 'yumex' )
        self.repo = Repository()
        self.profile = Profile()
        self.settings = Settings()
        self.debugMode = self.check_option( 'debug' )
        self.yw = None
        self.activeProfile = None
        self.profileGroup = None
        
        gtk.window_set_default_icon_from_file( self.PIXMAPS_PATH + '/yumex.png' )
	
        # Set limit for recent packages
        rdays = self.settings.get_value('recentdays')
        if rdays == None:
            recentdays = 14.0    
        else:
            recentdays = float(rdays)
        now = time.time()
        self.recentlimit = now-(recentdays*86400)
        
        self.progressDlg = ProgressDialog( self.GLADE_FILE, self.PIXMAPS_PATH )
        self.setup_gui()
        self.doGtkEvents()
        # Setup Action handler
        self.action = yumexAction(self.GLADE_FILE,self.progressDlg,self.console,self.status)
        # Get Yum data
        if self.check_option( "autorefresh" ):
            self.setup_yum()
            
            
    def doGtkEvents( self ):
        while gtk.events_pending():      # process gtk events
            gtk.main_iteration()
                      
    def cmdline_options( self ):
        """ Handle commmand line options """    
        parser = OptionParser()
        parser.add_option( "-d", "--debug", 
                        action="store_true", dest="debug", default=False, 
                        help="Debug mode" )
        parser.add_option( "-n", "--noauto", 
                        action="store_true", dest="noauto", default=False, 
                        help="No automatic refresh af program start" )

        ( options, args ) = parser.parse_args()
        self.cmd_options = options
        self.cmd_args = args
        #print self.cmd_options.debug
        #print self.cmd_args

    def check_option( self, option ):
        """ Check options in settings or command line"""
        rc = False
        if option == "debug":
            if self.settings.is_enabled( "debug" ) or self.cmd_options.debug:
                rc = True
        elif option == "autorefresh":
            if self.settings.is_enabled( "autorefresh" ) and not self.cmd_options.noauto:
                rc = True
        elif option == "filelists":
            if self.settings.is_enabled( "filelists" ):
                rc = True
        elif option == "changelog":
            if self.settings.is_enabled( "changelog" ):
                rc = True
        elif option == "autocleanup":
            if self.settings.is_enabled( "autocleanup" ):
                rc = True
        return rc

    def toolbar_handler(self,widget,page):
        if self.activePage in [1,2,3]:
            self.lastFilter = self.get_filter_text()
        if widget.get_active():
            self.notebook.set_current_page(page)
            self.activePage = page
            if page in [1,2,3] :
                curFilter = self.get_filter_text()
                if curFilter != self.lastFilter:
                    self.set_filter_text(self.lastFilter)

                    self.page_update_view()

    def group_row_handler( self, widget ):
        """ Handle selection of row in group view"""
        ( model, iterator ) = widget.get_selection().get_selected()
        if model != None and iterator != None: 
            grp = model.get_value( iterator, 0 )
            if not grp in ["Installed Groups","Avaliable Groups"]:
                print "Group : \n" 
                print self.yw.getGroupData(grp)
            
                    
                    
        
    def row_handler( self, widget, data ):
        """ Handle selection of row in install/update/remove view (Show Descriptions) """
        ( model, iterator ) = widget.get_selection().get_selected()
        if model != None and iterator != None:
            name = model.get_value( iterator, 1 )
            ver = model.get_value( iterator, 2 )
            arch = model.get_value( iterator, 5 )
            pkg = self.find_pkg( name, ver, arch )
            if pkg != None:
                desc = pkg.returnSimple( 'description' )
                if self.check_option("filelists"):
                    files = self.get_filelist(pkg)
                    text = "\n\nFiles:\n"
                    for f in files:
                        text = text + "  %s\n" % f
                    desc = desc + text
                    cl = self.get_changelog(pkg)
                    text = "\n\nChangelog:\n"
                    for l in cl:
                        text = text + l + "\n"
                    desc = desc + text
                if self.activePage==2:
                    pkg = self.find_current_installed( name, arch )
                    txt = _( "Installed version: %s (%s) \n\n" ) % ( pkg, arch )
                    txt = txt + desc
                    u = self.toUTF(txt)
                    data.get_buffer().set_text( u )
                else:
                    if desc != None:
                        u = self.toUTF(desc)
                        data.get_buffer().set_text( u ) 
                    else:
                        data.get_buffer().set_text( "" ) 
                    
    def get_filelist(self,pkg):    
        """ Get filelist from package object"""
        if self.activePage in [1,2]: # YumAvailablePackage
            files = pkg.returnFileEntries()
        else:  # YumInstalledPackage, get files from rpm header.
            files = []
            fn = pkg.hdr['basenames']
            dn = pkg.hdr['dirnames']
            for d,n in zip(dn,fn):
                files.append(d+n)
        return files
        
    def get_changelog(self,pkg):    
        """ Get changelog from package object"""
        cl = []
        if self.activePage in [1,2]: # YumAvailablePackage
            clog = pkg.returnChangelog()
            for tim,name,text in clog:
                cl.append(str(time.ctime(float(tim)))+" "+name+"\n"+text)
        else:  # YumInstalledPackage, get files from rpm header.
            cltime = pkg.hdr['changelogtime']
            clname = pkg.hdr['changelogname']
            cltext = pkg.hdr['changelogtext']
            for tim,name,text in zip(cltime,clname,cltext):
                cl.append(str(time.ctime(tim))+" "+name+"\n"+text)
        return cl
        
    def on_recent_click( self, col ): 
        if self.activePage in [1,3]:
            self.RecentState[self.activePage] = not self.RecentState[self.activePage]
            #print "recentstate : %s" % self.RecentState
            self.page_update_view()
 
    def on_button( self, widget, data ):
        """ Button Handler """
        #print "%s was pressed" % data
        if data == "INSTALL":
            self.action.set_selected(self.get_selected())
            if not self.action.install_packages():
                self.setup_yum(True)
            else:
                self.setup_yum(False)
        elif data == "UPDATE":
            self.action.set_selected(self.get_selected())
            if not self.action.update_packages():
                self.setup_yum(True)
            else:
                self.setup_yum(False)
        elif data == "REMOVE":
            self.action.set_selected(self.get_selected())
            if not self.action.remove_packages():
                self.setup_yum(True)
            else:
                self.setup_yum(False)
        elif data == "INST_DESELECT":
            self.installView.deselect_all()
        elif data == "RMV_DESELECT":
            self.removeView.deselect_all()
        elif data == "UPD_SELECT":
            self.updateView.select_all( )
        elif data == "REPO_REFRESH":
            self.setup_yum()            
        elif data == "REPO_DESELECT":
            self.repoView.deselect_all()
            

    def on_menu( self, widget, data ):
        """ Menu Handler """
        #print "Menu %s was activated" % data
        if data == "QUIT":
            self.delete_event()
        elif data == "EDIT":
            editrepo = EditRepo( self.GLADE_FILE )
            editrepo.show()
        elif data == "PREF":
            self.doPref()
        elif data == "EDIT_EXCLUDE":
            exclude_dialog( self.GLADE_FILE )
            self.settings = Settings()
        elif data == "ABOUT":
            ver = "Yum Extender %s - Build : %s" % ( self.VERSION, self.BUILD )
            about_dialog( self.GLADE_FILE, ver )
        elif data == "FILTER":
            self.doFilter()
        elif data == "PRO_SAVE":
            self.save_profile()
        elif data == "PRO_NEW":
            self.new_profile()
        elif data == "REPO_INST":
            RepoInstaller( self.GLADE_FILE, self.repo )
            self.setup_repoview()
            self.on_profile( self.profile.get_active() )

    def on_column_click( self, col ):
         print "click"

    def on_profile( self, data ):
        """ Profiles Menu Handler """
        self.set_status( _( "Selected the %s profile" ) % data )
        keys = self.profile.get_profile( data )
        if keys != False:
            self.repoView.select_by_keys( keys, False )
            self.set_main_title( data )
            self.activeProfile = data
        else:
            self.set_status( _( "Profile %s not found" ) % data )
        

    def delete_event( self, widget=None, event=None, data=None ):    
        if self.activeProfile != None:
            self.profile.set_active( self.activeProfile )
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__
        gtk.main_quit()
        sys.exit( 0 )         # Terminate Program

        return False
        
    def on_notebook_toggled( self, widget, column, model ):
        """ Notebook list checkbox handler """
        col = model[0]
        row = col[int( column )]
        row[0] = not row[0]
        return

    def on_repo_toggled( self, widget, column, model ):
        """ Repo list checkbox handler """
        col = model[0]
        row = col[int( column )]
        if row[1] in ( 'base', 'updates-released' ):
            state = None
            if self.base_row != -1:
                row_base =  col[self.base_row]
                state = not row_base[0]
                row_base[0] = state
            if self.updates_row != -1:
                row_updates =  col[self.updates_row]
                if state == None:
                    state = not row_updates[0]
                row_updates[0] = state
        else:
            row[0] = not row[0]        
        return


    def event_connect( self, widget, event, func, para=None ):
        """ connect callback function to event on widget """
        if type( widget ) == type( "" ):
            wid = self.get_widget( widget )
            if para == None:
                wid.connect( event, func )
            else:
                wid.connect( event, func, para )
            return wid
        else:
            widget.connect( event, func )
            return None


    def save_profile( self ):
        if self.activeProfile != None:
            repos = self.get_selected( 0 )
            if self.profile.set_profile( self.activeProfile, repos ):
                self.set_status( _( "Profile : %s saved ok" ) % self.activeProfile )
            else:
                self.set_status( _( "Profile : %s save failed" ) % self.activeProfile )
        else:
            self.set_status( _( "No profile selected" ) )

    def new_profile( self ):
        name = input_box( _( "Create New Profile" ), _( "_Profile name:" ) )
        if name != None:
            repos = self.get_selected( 0 )
            if self.profile.add_profile( name, repos ):
                self.add_to_profile_menu( name )
                self.activeProfile = name
                self.set_main_title( name )
                self.set_status( _( "Profile : %s created" ) % name )
            else:
                self.set_status( _( "Profile : %s creation failed" ) % name )

    def get_filter_text( self ):
        if self.activePage == 1:
            wid = self.instFilter
        elif self.activePage == 2:
            wid = self.updFilter
        else:
            wid = self.remFilter
        txt = wid.get_text()
        return txt

    def set_filter_text( self,text ):
        if self.activePage == 1:
            wid = self.instFilter
        elif self.activePage == 2:
            wid = self.updFilter
        else:
            wid = self.remFilter
        txt = wid.set_text(text)
        return txt


    def doPref( self ):
        Preferences( self.GLADE_FILE )
        self.settings.load() #Reload settings
    
    def doFilter( self ):
        if self.activePage in [1, 2, 3]:
            self.nameFilter = self.get_filter_text()
            self.page_update_view()

    def page_update_view( self ):
        """ Update View on Install,update,remove Notebook page """
        if self.activePage == 1:
                self.update_page( "INSTALL" )
        elif self.activePage == 2:
                self.update_page( "UPDATE" )
        elif self.activePage == 3:
                self.update_page( "REMOVE" )    

    def page_reselect( self, page, keys ):
        model = None
        if page == 1:
            model = self.installView.select_by_keys(keys)
        elif page == 2:
            model = self.updateView.select_by_keys(keys)
        elif page == 3:
            model = self.removeView.select_by_keys(keys)

    def get_selected( self, page=None ):
        selected = []
        if page == None:
            page = self.activePage
        if page == 1:
            selected = self.installView.get_selected()
        elif page == 2:
            selected = self.updateView.get_selected()
        elif page == 3:
            selected = self.removeView.get_selected()
        else:          
            selected = self.repoView.get_selected(False)
        return selected

    def get_notselected( self ):
        notselected = self.repoView.get_notselected()          
        return notselected
      
    def get_widget( self, widget ):
        """ get Glade Widget """
        return self.wTree.get_widget( widget )
        
    def set_main_title( self, extra = None ):
        if extra == None:
            self.window.set_title( "Yum Extender %s" % self.VERSION )
        else:
            self.window.set_title( "Yum Extender %s - %s Profile" % ( self.VERSION, extra ) )
    
    def on_repo_popup( self, label ):
        ( model, iterator ) = self.repoVid.get_selection().get_selected()       
        if model != None and iterator != None:
            activeRepo = model.get_value( iterator, 1 )
            if label == "_New":
                editrepo = EditRepo( self.GLADE_FILE,activeRepo,isNew=True)
                self.repo.update()
                self.setup_repoview()
                self.on_profile( self.profile.get_active() )                
            elif label == "_Edit":
                editrepo = EditRepo( self.GLADE_FILE,activeRepo)
            elif label == "_Delete":
                text = _(" Do you want to delete the <b>%s</b> repository?") % activeRepo
                rc = warning_box( _("Delete Repository"), text)
                if rc == "OK":
                    self.repo.delete_repo(activeRepo)
                    self.setup_repoview()
                    self.on_profile( self.profile.get_active() )                    
            elif label == "_Install":
                RepoInstaller( self.GLADE_FILE, self.repo )
                self.setup_repoview()
                self.on_profile( self.profile.get_active() )
    
    def init_repo_popup(self):
        popup_menu = gtk.Menu()
        self.add_to_menu(popup_menu,"_New",self.on_repo_popup)
        self.add_to_menu(popup_menu,"_Edit",self.on_repo_popup)
        self.add_to_menu(popup_menu,"_Delete",self.on_repo_popup)
        self.add_to_menu(popup_menu,"_Install",self.on_repo_popup)
        return popup_menu
        
    def add_to_menu(self,menu,label,handler):
        item = gtk.MenuItem(label)
        item.show()
        menu.append(item)
        item.connect_object("activate",handler,label)
            
    def add_to_radiomenu(self,menu,label,handler):
        if self.profileGroup == None:
            item = gtk.RadioMenuItem(None, label, False)
            self.profileGroup = item
        else:
            item = gtk.RadioMenuItem(self.profileGroup, label, False)
        if self.profile.get_active() == label:
            item.set_active(True)
        item.show()
        menu.append(item)
        item.connect_object("activate",handler,label)
    
    def on_view_click(self,treeview,event):
        #print event.button
        if event.button == 3: # Right Click
            x = int(event.x)
            y = int(event.y)
            time = event.time
            pthinfo = treeview.get_path_at_pos(x, y)
            if pthinfo != None:
                path, col, cellx, celly = pthinfo
                treeview.grab_focus()
                treeview.set_cursor( path, col, 0)
                self.repo_popup.popup( None, None, None, event.button, time)
    
    def setup_gui( self ):
        """ Setup GUI """
        windowname="main"
        self.wTree=gtk.glade.XML ( self.GLADE_FILE, windowname, domain='yumex' )
        self.window = self.event_connect( "main", "delete_event", self.delete_event )
        self.set_main_title()
        self.window.present()
        self.tooltip = gtk.Tooltips() # Tooltip Object
        
        # Setup Vertical Toolbar
        self.content = self.wTree.get_widget( "content" )
        self.button2 = self.createButton( _("Update"), "button-update.png", None, 2 )
        self.button1 = self.createButton( _("Install"), "button-install.png", self.button2, 1 )
        self.button3 = self.createButton( _("Remove"), "button-remove.png", self.button2, 3 )
        self.button0 = self.createButton( _("Repos"), "button-repo.png", self.button2, 0 )
        self.button4 = self.createButton( _("Output"), "button-output.png", self.button2, 4 )
        #self.button5 = self.createButton( _("Groups"), "button-output.png", self.button2, 5 )
        self.leftEventBox = self.wTree.get_widget( "leftEvent" )
        style = self.leftEventBox.get_style()
        bgcolor = gtk.gdk.Color( 65535, 65535, 65535 )
        self.leftEventBox.modify_bg( gtk.STATE_NORMAL, bgcolor )
        self.wTree.get_widget( 'pageImage0' ).set_from_file ( self.PIXMAPS_PATH + '/button-repo.png' )
        self.wTree.get_widget( 'pageImage1' ).set_from_file ( self.PIXMAPS_PATH + '/button-install.png' )
        self.wTree.get_widget( 'pageImage2' ).set_from_file ( self.PIXMAPS_PATH + '/button-update.png' )
        self.wTree.get_widget( 'pageImage3' ).set_from_file ( self.PIXMAPS_PATH + '/button-remove.png' )
        self.wTree.get_widget( 'pageImage4' ).set_from_file ( self.PIXMAPS_PATH + '/button-output.png' )
        #self.wTree.get_widget( 'pageImage5' ).set_from_file ( self.PIXMAPS_PATH + '/button-update.png' )

        
        # Setup Notebook
        self.notebook = self.get_widget( "notebook" )
        self.notebook.set_show_tabs( False )
        self.notebook.set_current_page( 2 )
        self.activePage = 2
      
        # Setup Button Handler
        #  Repo Page
        self.event_connect( "repoRefresh", "clicked", self.on_button, "REPO_REFRESH" )
        self.event_connect( "repoDeSelect", "clicked", self.on_button, "REPO_DESELECT" )
        self.event_connect( "repoInstall", "clicked", self.on_menu, "REPO_INST" )

        #  Install Page
        self.event_connect( "instInstall", "clicked", self.on_button, "INSTALL" )
        self.event_connect( "instDeSelect", "clicked", self.on_button, "INST_DESELECT" )
        #  Update Page
        self.event_connect( "updUpdate", "clicked", self.on_button, "UPDATE" )
        self.event_connect( "updSelectAll", "clicked", self.on_button, "UPD_SELECT" )
        #  Remove Page
        self.event_connect( "rmvRemove", "clicked", self.on_button, "REMOVE" )
        self.event_connect( "rmvDeSelect", "clicked", self.on_button, "RMV_DESELECT" )
        
        # Setup Menu Handler
        self.event_connect( "fileQuit", "activate", self.on_menu, "QUIT" )
        self.event_connect( "fileRepoInst", "activate", self.on_menu, "REPO_INST" )
        self.event_connect( "editPref", "activate", self.on_menu, "PREF" )
        self.event_connect( "editExclude", "activate", self.on_menu, "EDIT_EXCLUDE" )
        self.event_connect( "proNew", "activate", self.on_menu, "PRO_NEW" )
        self.event_connect( "proSave", "activate", self.on_menu, "PRO_SAVE" )
        self.event_connect( "helpAbout", "activate", self.on_menu, "ABOUT" )

        # Setup filter widget

        self.instFilter = self.event_connect( "instFilter", "activate", self.on_menu, "FILTER" )
        self.updFilter = self.event_connect( "updFilter", "activate", self.on_menu, "FILTER" )
        self.remFilter = self.event_connect( "remFilter", "activate", self.on_menu, "FILTER" )
        self.instFilter.set_text( "" )
        self.updFilter.set_text( "" )
        self.remFilter.set_text( "" )

        # Set Description TextViews
        self.instDesc =  self.get_widget( "instDesc" )
        self.instDesc.get_buffer().set_text( "" )
        self.remDesc =  self.get_widget( "remDesc" )
        self.remDesc.get_buffer().set_text( "" )
        self.updDesc =  self.get_widget( "updDesc" )
        self.updDesc.get_buffer().set_text( "" )

        # Add Profiles to Profiles menu.        
        self.proMenu = self.get_widget( "profileMenu" )
        for p in self.profile.get_list():
            self.add_to_profile_menu( p )
            
        # Repository View
        self.repoVid = self.get_widget( "viewRepo" )
        headers = [_( "Repository" ), _( "File" ), "", "", "", "" ]
        self.repoView = pkgView(self.repoVid,headers, self.on_repo_toggled)
        self.repo_popup = self.init_repo_popup()
        self.repoVid.connect("button_press_event",self.on_view_click)
        # Install View
        self.installVid = self.get_widget( "viewInstall" )
        self.installVid.connect( "cursor_changed", self.row_handler, self.instDesc )
        self.installView = pkgView(self.installVid,new_handler=self.on_recent_click)
        # Update View
        self.updateVid = self.get_widget( "viewUpdate" )
        self.updateVid.connect( "cursor_changed", self.row_handler, self.updDesc )
        self.updateView = pkgView(self.updateVid)
        # Remove view
        self.removeVid = self.get_widget( "viewRemove" )
        self.removeVid.connect( "cursor_changed", self.row_handler, self.remDesc )
        self.removeView = pkgView(self.removeVid,new_handler=self.on_recent_click)      
        # Group View
        #self.grpVid = self.get_widget( "viewGroups" )
        #self.grpView = grpView(self.grpVid)
        #self.grpVid.connect( "cursor_changed", self.group_row_handler)

        # Setup Console
        #self.progress = self.get_widget("progress")
        self.output = self.get_widget( "viewOutput" )
        sw = self.get_widget( "outputSW" )
        self.console = NoteBookConsole( sw, self.output, self.debugMode )
        self.status = self.get_widget( "status" )
        self.setup_repoview()

        # Reload Last active Profile
        self.on_profile( self.profile.get_active() )


    def createButton( self, text, icon, group, page ):
          button = gtk.RadioButton( group )
          button.connect( "clicked", self.toolbar_handler, page )

          button.set_relief( gtk.RELIEF_NONE )
          button.set_mode( False )

          p = gtk.gdk.pixbuf_new_from_file( self.PIXMAPS_PATH+"/"+icon )
          pix = gtk.Image()
          pix.set_from_pixbuf( p )
          pix.show()

          buttonLabel = gtk.Label( "" )
          buttonLabel.set_markup( '<span foreground="#000000">%s</span>' % text )
          buttonLabel.show()
          buttonVBox = gtk.VBox()
          buttonVBox.pack_start( pix )
          buttonVBox.pack_start( buttonLabel )
          button.add( buttonVBox )
          button.show()
          buttonVBox.show()
          self.content.pack_start( button, False )
          return button

    def add_to_profile_menu( self, name ):
        sub = self.proMenu.get_submenu()
        self.add_to_radiomenu(sub, name, self.on_profile)
    
    def set_status( self, text ):
        context_id = self.status.get_context_id( "Status" )
        self.status.push( context_id, text )
   

    def setup_combo_entry( self, entry ):
        model = gtk.ListStore( gobject.TYPE_STRING )
        entry.set_model( model )
        return model

    def setup_repoview( self ):
        repos = self.repo.get_names_sorted()
        data = []
        i = 0
        for r in repos:
            if r == 'base':
                self.base_row = i
            if r == 'updates-released':
                self.updates_row = i      
            el = ( False, r, self.repo.get_repo_filename( r ), "", "", "", "",False )
            data = data + [el]
            i = i+1 
        self.repoView.populate_view( data )

    def populate_combo( self, model, data ):
        """ Populate a Notebook page with data """
        model.clear()
        for d in data:
            iterator = model.append()
            model.set_value( iterator, 0, d )

    def find_pkg( self, name, ver, arch ):
        page = self.activePage
        rc=None
        if page in [1,2,3]:
            if page == 1:
                pkglist = self.yw.available
            elif page == 2:
                pkglist = self.yw.updates
            elif page == 3:
                pkglist = self.yw.installed
            for pkg in pkglist:
                if pkg.name == name:
                    v = pkg.printVer()
                    #print "ver : (%s) (%s)" % (ver,v)
                    #print "arch : (%s) (%s)" % (arch,pkg.arch)
                    if pkg.arch == arch and v == ver:
                        rc = pkg
        return rc
                
    def find_current_installed( self, name, arch ):
        pkglist = self.yw.installed
        rc = None
        for pkg in pkglist:
            if pkg.name == name:
                if pkg.arch == arch:
                    rc = pkg.printVer()
        return rc

    def update_page( self, no ):
        self.set_status( _("Updating %s view, please wait... ") % no.lower() )
        self.doGtkEvents()
        if no == "INSTALL":
            data = self.build_page_data( self.yw.available)
            self.doGtkEvents()
            self.instDesc.get_buffer().set_text( "" )
            self.installView.populate_view( data )
        elif no == "UPDATE":
            data = self.build_page_data( self.yw.updates)
            self.doGtkEvents()
            self.updDesc.get_buffer().set_text( "" )
            self.updateView.populate_view( data )
        elif no == "REMOVE":
            data = self.build_page_data( self.yw.installed)
            self.doGtkEvents()
            self.remDesc.get_buffer().set_text( "" )
            self.removeView.populate_view( data )
        elif no == "GROUPS":
            data = self.yw.getGroupListData()
            self.doGtkEvents()
            self.grpView.populate_view(data)
        self.set_status( "" )

    def build_page_data( self, lst,data = None ):
        if data == None:
            data = []
        if self.nameFilter == "":
            for pkg in lst:
                if self.activePage in [1,3] and self.RecentState[self.activePage]: 
                    if not self.is_recent(pkg):
                        pass
                    else: 
                        data = data + self.get_row_data( pkg )
                else: 
                    data = data + self.get_row_data( pkg )
        else:
            flt = self.nameFilter.upper()
            arch_flt = self.get_arch_filter( flt )
            for pkg in lst:
                if self.check_filter( flt, pkg, arch_flt ):                  
                    data = data + self.get_row_data( pkg )
        data.sort()
        return data

    def get_row_data( self, pkg ):
        ver = pkg.printVer()
        size = self.yw.format_number( float( pkg.size() ) )
        summary = self.toUTF(pkg.returnSimple( 'summary' ).splitlines()[0])
        new = self.is_recent(pkg)
        el = ( False, pkg.name, ver, summary, pkg.repoid, pkg.arch, size, new )
        return [el]

    def toUTF(self,txt):
        rc=""
        try:
            rc = unicode(txt,'utf-8')
        except UnicodeDecodeError, e:
            #print e
            #print txt
            rc = unicode(txt,'iso-8859-1')
        return rc


    def is_recent(self, pkg):
        rc = False
        if pkg.repoid == "installed": # Installed packages dont have filetime
            ftime = int(pkg.returnSimple('installtime'))
            if ftime > self.recentlimit:
                rc = True            
        else:
            ftime = int(pkg.returnSimple('filetime'))
            if ftime > self.recentlimit:
                rc = True
        return rc
        
    def get_arch_filter( self, flt ):
        arch_re = re.compile( '(?<=ARCH=)\w+' )
        arch_rc = arch_re.search( flt )
        if arch_rc != None:
            print "Arch : %s" % arch_rc.group( 0 )
            arch_flt = arch_rc.group( 0 )
        else:
            arch_flt = None
        return arch_flt
        
    def check_filter( self, flt, pkg, arch_flt ):
        if self.activePage in [1,3] and self.RecentState[self.activePage]:
            if not self.is_recent(pkg):
                return False
        name = pkg.returnSimple( 'name' )
        summary = pkg.returnSimple( 'summary' )
        desc = pkg.returnSimple( 'description' )
        if name == None: name = ""
        if summary == None: summary = ""
        if desc == None: desc = ""
        keys = [name.upper(), summary.upper(), desc.upper()]
        if pkg.arch == None:
            arch = ""
        else:
            arch = pkg.arch.upper()
        words = flt.split( ' ' )
        # Check for filter words
        for word in words:
            if word.find( "=" ) <> -1:
                pass
            else:
                if not self.check_for_word( word, keys ):
                    return False
        # Check for Architechture filter
        if arch_flt == None:
            return True
        else:
            if arch == arch_flt:
                return True
            else:
                return False
            
    def check_for_word( self, word, keys ):
        for key in keys:
            if key.find( word ) <> -1:
                return True
        return False
        
    def get_options(self):
        check = ['filelists','autocleanup','debug']
        options = []
        for c in check:
            if self.check_option( c ):
                options.append(c)
        return options
   
         
    def setup_yum(self,save_selection = False):
        self.progressDlg.show()  
        self.progressDlg.set_mainLabel( _("Initializing Repositories") )        
        if save_selection:
            save_sel = self.get_selected()
            save_page = self.activePage        
        options = self.get_options()
        loglevel = 2
        if self.yw != None:
            del self.yw
        self.yw = yumexBase(self.console,self.progressDlg,loglevel,options)
        self.action.set_yumbase(self.yw)
        if self.yw.setup():
            self.yw.set_active_repos(self.get_selected( 0 ))
            if self.yw.load_repo_data():
                self.yw.build_package_lists()
                self.progressDlg.hide()  
                self.update_page( "REMOVE" )
                self.update_page( "INSTALL" )
                self.update_page( "UPDATE" )
                #self.update_page( "GROUPS" )
                
                #self.console.activate_progress(False)
                self.set_status( _( "Yum initialization complete" ) )
                #print "------------------------------------------------"
                #print gc.garbage
                if save_selection:
                    self.page_reselect( save_page, save_sel )
            else:
                self.progressDlg.hide()
                msg = _("Error in loading repository data\n\n")+self.yw.last_error
                dialogs.warning_box( _( "Error" ), msg )
        else:
            self.progressDlg.hide()
            msg = _("Error in yum setup\n\n")+self.yw.last_error
            dialogs.warning_box( _( "Error" ), msg )


def main():
    gtk.main()

if __name__ == "__main__":
    mainApp = MainApplication()
    main()



