#!/usr/bin/python -tt
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: dialogs.py,v 1.6.2.4 2005/10/15 16:59:55 timlau Exp $

"""Yum Extender Dialogs"""

import sys, os, gobject, glob, shutil, datetime
import gtk
import gtk.glade
import ConfigParser
import pygtk
from utils import *
from i18n import _

class Preferences:
    def __init__( self, xmlfile ):
        self.Tree=gtk.glade.XML ( xmlfile, 'pref' )
        self.conf_file = '/etc/yumex.conf'
        self.section = 'yumex'
        self.conf = ConfigParser.ConfigParser()
        self.conf.read( self.conf_file )             
        self.setup_gui()
        gtk.main()
                     

    def setup_gui( self ):
        self.win = self.Tree.get_widget( 'pref' )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.bOK = self.Tree.get_widget( "prefOK" )
        self.bOK.connect( "clicked", self.click, "OK" )
        self.autoRefresh = self.Tree.get_widget( "prefAutoRefresh" )
        self.autoRefresh.set_active( self.get_boolean( 'autorefresh' ) )
        self.debugMode = self.Tree.get_widget( "prefDebug" )
        self.debugMode.set_active( self.get_boolean( 'debug' ) )
        self.fileList = self.Tree.get_widget( "prefFilelists" )
        self.fileList.set_active( self.get_boolean( 'filelists' ) )
        self.autoCleanup = self.Tree.get_widget( "prefCleanup" )
        self.autoCleanup.set_active( self.get_boolean( 'autocleanup' ) )
        self.win.show() 
    
    def update_conf( self ):
        self.set_boolean( "autorefresh", self.autoRefresh.get_active() )
        self.set_boolean( "debug", self.debugMode.get_active() )
        self.set_boolean( "filelists", self.fileList.get_active() )
        self.set_boolean( "autocleanup", self.autoCleanup.get_active() )

    def get_value( self, option ):
        if self.conf.has_option( self.section, option ):
            return self.conf.get( self.section, option )
        else:
            return None

        
    def set_value( self, option, value="" ):
        if value != "":
            self.conf.set( self.section, option, value )
        else:
            if self.conf.has_option( self.section, option ):
                self.conf.remove_option( self.section, option )
    
    def set_boolean( self, option, state ):
        if state:
            self.conf.set( self.section, option, "1" )
        else:
            self.conf.set( self.section, option, "0" )

    def get_boolean( self, option ):
        if self.conf.has_option( self.section, option ):
            if self.conf.get( self.section, option ) == "1":
                return True
            else:
                return False
        else:
            return False
        
    def save( self ):
        f=open( self.conf_file, "w" )
        self.conf.write( f )
        f.close()        

    def quit( self, w=None, event=None ):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()

    def click( self, button, key ):
        if key == "OK":
            self.update_conf()
            self.save()
            self.quit()
        elif key == "PROXY":
            self.proxyState = self.useProxy.get_active()
            if self.proxyState:
                self.boxProxy.show()     
            else:
                self.boxProxy.hide()     
            

 



class ProgressDialog:
    """ Progress DialogClass """
    def __init__( self, xmlfile, gfxPath ):
        self.Tree=gtk.glade.XML ( xmlfile, 'progress' )
        self.win = self.Tree.get_widget( "progress" )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.bOK = self.Tree.get_widget( "progressOK" )
        self.bOK.connect( "clicked", self.click, "OK" )
        self.bCancel = self.Tree.get_widget( "progressCancel" )
        self.bCancel.connect( "clicked", self.click, "Cancel" )
        self.image = self.Tree.get_widget( "progressImage" )
        self.image.set_from_file( gfxPath+"/button-repo.png" )
        self.image.show()
        self.mainLabel = self.Tree.get_widget( "progressMainLabel" )
        self.mainLabel.set_text( "" )
        self.subLabel = self.Tree.get_widget( "progressSubLabel" )
        self.subLabel.set_text( "" )
        self.extraLabel = self.Tree.get_widget( "progressExtraLabel" )
        self.extraLabel.set_text( "" )
        self.etaLabel = self.Tree.get_widget( "progressETALabel" )
        self.etaLabel.set_text( "" )
        self.progress = self.Tree.get_widget( "progressBar" )
        self.progress.set_fraction( 0 )
        self.progress.set_text( " " )
        self.win.hide()

        
    def show( self ):
        self.win.show()
        
    def hide( self, clean=True ):
        self.win.hide()
        if clean:
            self.mainLabel.set_text( "" )
            self.subLabel.set_text( "" )
            self.extraLabel.set_text( "" )
            self.progress.set_fraction( 0 )
            self.progress.set_text( " " )
        
    def set_progress( self, frac, text=None ): 
        if frac >= 0 and frac <= 1:
            self.progress.set_fraction( frac )
        else:
            self.progress.set_fraction( 0 )
        if text != None:
            self.progress.set_text( text )
        
    def set_mainLabel( self, text ):
        self.mainLabel.set_markup( "<span size=\"large\"><b>%s</b></span>" % text )
        self.subLabel.set_text( "" )
        self.extraLabel.set_text( "" )
        
    def set_subLabel( self, text ):
        #print text
        self.subLabel.set_markup( "%s" % text )
        #self.subLabel.set_text( "%s" % text )
        self.extraLabel.set_text( "" )
        
    def set_extraLabel( self, text ):
        self.extraLabel.set_markup( "<span size=\"small\">%s</span>" % text )
        
    def set_etaLabel( self, text ):
        self.etaLabel.set_markup( "<span size=\"small\">%s</span>" % text )
                     
    def quit( self, w=None, event=None ):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()

    def click( self, button, key ):
        if key == "OK":
            self.ret = True
            self.hide()
        else:
            self.ret = False
            self.hide()
        #self.quit()
        

class RepoInstaller:
    def __init__( self, xmlfile, repo ):
        self.Tree=gtk.glade.XML ( xmlfile, 'repoInstall' )
        self.repo = repo
        self.yumex_repo_etcfile = '/etc/yumex.repoinst'
        self.yumex_repos = ConfigParser.ConfigParser()
        if os.path.exists( self.yumex_repo_etcfile ):
             self.yumex_repos.read( self.yumex_repo_etcfile )
        self.setup_gui()
        gtk.main()
                     
    def on_toggled( self, widget, column, model ):
        """ list checkbox handler """
        col = model[0]
        row = col[int( column )]
        row[0] = not row[0]
        return

    def setup_gui( self ):
        self.win = self.Tree.get_widget( 'repoInstall' )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.bAdd = self.Tree.get_widget( "repoInstAdd" )
        self.bAdd.connect( "clicked", self.click, "ADD" )
        self.bClose = self.Tree.get_widget( "repoInstClose" )
        self.bClose.connect( "clicked", self.click, "CLOSE" )
        self.repoView = self.Tree.get_widget( "repoInstTree" )
        self.repoModel = self.setup_view( self.repoView )
        self.update_list()
        self.win.show() 

    def update_list( self ):
        data = self.getRepoData()
        self.populate_view( self.repoModel, data )

    def addRepos( self ):
        sel = self.getSelected( self.repoModel )
        repos = self.repo.get_names_sorted()
        txt = _("Do you want to add these repositories \n\n")
        for s in sel:
            if s not in repos: # only add not installed ones  
                txt = txt + "%s\n" % s
        rc = warning_box( _("Add Repositories"), txt )
        if rc == "OK":
            for s in sel:
                if s not in repos: # only add not installed ones  
                    ( title, name, urltype, url, enable, gpg, gpgkey ) = self.get_repo_info( s )
                    self.repo.create_new( title, name, urltype, url, enable, gpg, gpgkey )
            self.update_list()
            
    def get_repo_info( self, nam ):
        title = nam
        name = self.yumex_repos.get( title, "name" )
        if self.yumex_repos.has_option( title, "baseurl" ):
            url = self.yumex_repos.get( title, "baseurl" )
            urltype = "baseurl"
        else:
            url = self.yumex_repos.get( title, "mirrorlist" )
            urltype = "mirrorlist"
        enable = self.yumex_repos.get( title, "enabled" )
        if self.yumex_repos.has_option( title, "gpgcheck" ):
            gpg = self.yumex_repos.get( title, "gpgcheck" )
        else:
            gpg = None
        if self.yumex_repos.has_option( title, "gpgkey" ):
            gpgkey = self.yumex_repos.get( title, "gpgkey" )
        else:
            gpgkey = None
        return ( title, name, urltype, url, enable, gpg, gpgkey )

    def getSelected( self, model ):
        selected = []
        for elem in model:
            state = elem[0]
            name = elem[1]
            if state:
                selected = selected + [name]
        return selected


    def getRepoData( self ):
        data = []
        repos = self.repo.get_names_sorted()
        yrepos = self.yumex_repos.sections()
        for r in yrepos:
            if r not in repos:
                ( tag, name, urltype, url, enable, gpg, gpgkey ) = self.get_repo_info( r )
                elem = ( False, r, name )
                data = data + [elem]
        data.sort()
        return data
        

    def setup_view( self, view ):
        """ Create Notebook list for single page  """
        model = gtk.ListStore( 'gboolean', gobject.TYPE_STRING, gobject.TYPE_STRING )
        view.set_model( model )
        cell1 = gtk.CellRendererToggle()
        cell1.set_property( 'activatable', True )
        column1 = gtk.TreeViewColumn( "    ", cell1 )
        column1.add_attribute( cell1, "active", 0 )
        view.append_column( column1 )
        cell2 = gtk.CellRendererText()
        column2 = gtk.TreeViewColumn( "Tag", cell2, text=1 )
        view.append_column( column2 )
        cell3 = gtk.CellRendererText()
        column3 = gtk.TreeViewColumn( "Name", cell3, text=2 )
        view.append_column( column3 )
        cell1.connect( "toggled", self.on_toggled, ( model, column1 ) )
        return model

    def populate_view( self, model, data ):
        """ Populate view with data """
        model.clear()
        for col1, col2, col3 in data:
            iterator = model.append()
            model.set_value( iterator, 2, col3 )
            model.set_value( iterator, 1, col2 )
            model.set_value( iterator, 0, col1 )

    def quit( self, w=None, event=None ):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()

    def click( self, button, key ):
        if key == "ADD":
            self.addRepos()
        else:
            self.quit()

class EditRepo:
    def __init__( self, xmlfile, thisrepo, isNew= False ):
        # Setup Glade Widgets
        self.xml = gtk.glade.XML( xmlfile, "repowin" )
        self.window = self.xml.get_widget( "repowin" )
        self.ok = self.xml.get_widget( "repoOK" )        
        self.cancel = self.xml.get_widget( "repoCancel" )        
        self.optEnabled = self.xml.get_widget( "repoEnabled" )
        self.optGPG = self.xml.get_widget( "repoGPG" )    
        self.optMirrorList=self.xml.get_widget( "repoMirrorlist" )        
        self.optBaseurl = self.xml.get_widget( "repoBaseurl" )        
        self.title = self.xml.get_widget( "repoTitle" )    
        self.name = self.xml.get_widget( "repoName" )     
        self.GPGKey = self.xml.get_widget( "repoGPGKey" )              
        self.URL = self.xml.get_widget( "repoURL" )        
        
        # Other Member

        self.notSaved = False
        
        # Setup Event Handlers    
        self.window.connect( "delete_event", self.on_Delete )
        self.window.connect( "frame_event", self.on_Delete )
        # Buttons Events
        self.ok.connect( "clicked", self.on_OK )
        self.cancel.connect( "clicked", self.on_Cancel )
        self.repo = Repository()        
        self.window.present()
        if not isNew:
            self.repo.set_active( thisrepo )
            self.update_fields()    
        else:
            self.new_repo()

    def show( self ):
        #print "EditRepo: show"
        self.window.present()
    
    def hide( self ):
        #print "EditRepo: hide"
        self.window.hide()
        
    def update_fields( self ):
        self.optEnabled.set_active( self.repo.get_flag( "enabled" ) )
        self.optGPG.set_active( self.repo.get_flag( "gpgcheck" ) )
        if self.repo.isBaseUrl():
            self.optMirrorList.set_active( False )
            self.optBaseurl.set_active( True )
            urls = self.repo.get_option( "baseurl" )
        else:
            self.optMirrorList.set_active( True )
            self.optBaseurl.set_active( False )
            urls = self.repo.get_option( "mirrorlist" )
        self.orgTitle = self.repo.activeRepo
        self.title.set_text( self.orgTitle )
        self.name.set_text( self.repo.get_option( "name" ) )
        gpgkey = self.repo.get_option( "gpgkey" )
        if gpgkey != None:
             self.GPGKey.set_text( gpgkey )
        else:
             self.GPGKey.set_text( "" )
             
        self.writelines( self.URL, urls )
        self.notSaved = True;
        
    def writelines( self, view, lines ):
        buf = view.get_buffer()
        buf.set_text( "" )
        start = buf.get_start_iter()
        buf.insert( start, lines )
                
    def on_repomenu_activate( self, repo ):
        self.checkSave()
        self.repo.set_active( repo )
        self.update_fields()    
    
    def on_OK( self, key ):
        self.checkSave()
        self.window.hide()
    
    def on_Cancel( self, key ):
        self.window.hide()
        
    def on_Delete( self, event, signal ):
        self.checkSave()
        self.hide()
        event.emit_stop_by_name( "delete_event" )
        return True

    def new_repo( self):
        title=input_box( _("New Repository"), _("_New repository tag name:") )
        #print title
        if title != None and title.strip() != "":
            if self.repo.create_new( title, _("New Repository"), "baseurl" ):
                self.update_fields()    
            else:
                warning_box( _("Error"), _("Repository Creation Failed") )

        
    def readlines( self ):
        lines = []
        buf = self.URL.get_buffer()
        start = buf.get_start_iter()
        end = buf.get_end_iter()
        lines = buf.get_text( start, end )
        return lines
        
        
    def update_repo( self ):
        self.repo.set_option( "name", self.name.get_text() )
        lines = self.readlines()
        if self.optBaseurl.get_active():                    # Check for baseurl
            if not self.repo.isBaseUrl():                # check mirrorlist -> baseurl    
                self.repo.remove_option( "mirrorlist" )    #        remove mirrorlist
            self.repo.set_option( "baseurl", lines )        #    add baseurl    
        else:                                                        # Mirrorlist, only one line    
            if self.repo.isBaseUrl():                            # check baseurl -> mirrorlist
                self.repo.remove_option( "baseurl" )        #        remove baseurl
            line = lines.split( "\n" )[0]                    # Only use first url.
            self.repo.set_option( "mirrorlist", line )    # add mirrorlist        
        self.repo.set_flag( "enabled", self.optEnabled.get_active() )
        self.repo.set_flag( "gpgcheck", self.optGPG.get_active() )
        gpgkey = self.GPGKey.get_text()
        if gpgkey != "":
             self.repo.set_option( "gpgkey", gpgkey )

    def checkSave( self ):
        if self.isChanged():
            if self.notSaved:
                rc=warning_box( "Save Changes", "Do you want to save changes ?" )
                if rc != None:                    
                    self.update_repo()
                    #self.repo.list_active()
                    self.repo.save()

        
    def isChanged( self ):
        urls = self.readlines()
        if self.name.get_text() != self.repo.get_option( "name" ):
            Changed = True
        elif  self.optBaseurl.get_active() and urls != self.repo.get_option( "baseurl" ):
            Changed = True
        elif  self.optMirrorList.get_active() and urls != self.repo.get_option( "mirrorlist" ):
            Changed = True
        elif self.optBaseurl.get_active() != self.repo.isBaseUrl():
            Changed = True
        elif self.optEnabled.get_active() != self.repo.get_flag( "enabled" ):
            Changed = True
        elif self.optGPG.get_active() != self.repo.get_flag( "gpgcheck" ):
            Changed = True
        elif self.GPGKey.get_text() != self.repo.get_option( "gpgkey" ):
            if self.GPGKey.get_text() <> "":
                 Changed = True
            else:
                 Changed = False
        else:
            Changed = False
        return Changed            
        

class FileChooser( gtk.FileSelection ):
    def __init__( self, modal=True, multiple=True, pattern="*.*" ):
        gtk.FileSelection.__init__( self )
        self.multiple = multiple
        self.connect( "destroy", self.quit )
        self.connect( "delete_event", self.quit )
        if modal:
            self.set_modal( True )
        self.cancel_button.connect( 'clicked', self.quit )
        self.ok_button.connect( 'clicked', self.ok_cb )
        if multiple:
            self.set_select_multiple( True )
            ## self.hide_fileop_buttons()
        self.complete( pattern )
        self.ret = None
    def quit( self, *args ):
        self.hide()
        self.destroy()
        gtk.main_quit()
    def ok_cb( self, b ):
        if self.multiple:
            self.ret = self.get_selections()
        else:
            self.ret = self.get_filename()
        self.quit()

def file_sel_box( title="Browse", modal=False, multiple=True, pattern="*.*" ):
    win = FileChooser( modal=modal, multiple=multiple, pattern=pattern )
    win.set_title( title )
    win.show()
    gtk.main()
    return win.ret

def file_open_box( modal=True, pattern="*.*" ):
    return file_sel_box( "Open", modal=modal, multiple=False, pattern=pattern )
def file_save_box( modal=True, pattern="*.*" ):
    return file_sel_box( "Save As", modal=modal, multiple=False, pattern=pattern )


class EntryDialog( gtk.Dialog ):
    def __init__( self, message="", default_text='', modal= True ):
        gtk.Dialog.__init__( self )
        self.connect( "destroy", self.quit )
        self.connect( "delete_event", self.quit )
        if modal:
            self.set_modal( True )
        box = gtk.VBox( spacing=6 )
        box.set_border_width( 5 )
        self.set_border_width( 5 )
        self.vbox.set_spacing( 2 )
        self.vbox.pack_start( box )
        box.show()
        if message:
            label = gtk.Label( message )
            label.set_alignment( 0.0, 0.5 )
            label.set_use_underline( True )
            box.pack_start( label )
            label.show()
        self.set_has_separator( False )
        self.entry = gtk.Entry()
        self.entry.set_text( default_text )
        box.pack_start( self.entry )
        self.entry.show()
        self.entry.grab_focus()
        label.set_mnemonic_widget( self.entry )
        button = gtk.Button( "Cancel", "gtk-cancel", True )
        button.connect( "clicked", self.quit )
        button.set_flags( gtk.CAN_DEFAULT )
        self.action_area.pack_start( button )
        button.show()
        button = gtk.Button( "OK", "gtk-ok", True )
        button.connect( "clicked", self.click )
        button.set_flags( gtk.CAN_DEFAULT )
        self.action_area.pack_start( button )
        button.show()
        button.grab_default()
        self.ret = None
    def quit( self, w=None, event=None ):
        self.hide()
        self.destroy()
        gtk.main_quit()
        
    def click( self, button ):
        self.ret = self.entry.get_text()
        self.quit()

class infoDialog( gtk.Dialog ):
    def __init__( self, message="", modal= True ):
        gtk.Dialog.__init__( self )
        self.connect( "destroy", self.quit )
        self.connect( "delete_event", self.quit )
        if modal:
            self.set_modal( True )
        box = gtk.VBox( spacing=6 )
        box.set_border_width( 5 )
        self.set_border_width( 5 )
        self.vbox.set_spacing( 2 )
        self.vbox.pack_start( box )
        self.set_has_separator( False )
        box.show()
        if message:
            label = gtk.Label()
            label.set_alignment( 0.0, 0.5 )
            box.pack_start( label )
            label.set_markup(message)
            label.show()
        button = gtk.Button( "Cancel", "gtk-cancel", True )
        button.connect( "clicked", self.quit )
        button.set_flags( gtk.CAN_DEFAULT )
        self.action_area.pack_start( button )
        button.show()
        button = gtk.Button( "OK", "gtk-ok", True )
        button.connect( "clicked", self.click )
        button.set_flags( gtk.CAN_DEFAULT )
        self.action_area.pack_start( button )
        button.show()
        button.grab_default()
        self.ret = None
    def quit( self, w=None, event=None ):
        self.hide()
        self.destroy()
        gtk.main_quit()
    def click( self, button ):
        lab = button.get_label()
        if lab == "gtk-ok":
            self.ret = "OK"
        else:
            self.ret = "Cancel"
        self.quit()


def input_box( title="Input Box", message="", default_text='', 
          modal= True ):
     dlg = EntryDialog( message, default_text, modal=modal )
     dlg.set_title( title )
     dlg.show()
     gtk.main()
     return dlg.ret

def warning_box( title="Warning", message="", modal= True ):
     dlg = infoDialog( message, modal=modal )
     dlg.set_title( title )
     dlg.show()
     gtk.main()
     return dlg.ret

class FilterDialog:
    def __init__( self, xmlfile, flt ):
        self.filterTree=gtk.glade.XML ( xmlfile, 'filter' )
        self.win = self.filterTree.get_widget( "filter" )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.entry = self.filterTree.get_widget( "fltFilter" )
        self.entry.set_text( flt )
        self.bApply = self.filterTree.get_widget( "fltApply" )
        self.bApply.connect( "clicked", self.click )
        self.ret = flt
        self.win.show() 
                     
    def quit( self, w=None, event=None ):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()
    def click( self, button ):
        self.ret = self.entry.get_text()
        self.quit()

def filter_dialog( xml, flt ):
    dlg = FilterDialog( xml, flt )
    gtk.main()
    return dlg.ret

class ConfimationDialog:
    def __init__( self, xmlfile, action, pkg, dep, size ):
        self.filterTree=gtk.glade.XML ( xmlfile, 'confirmation' )
        self.win = self.filterTree.get_widget( "confirmation" )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.action = self.filterTree.get_widget( "confAction" )
        tit = "<span size='x-large'> Packages to %s </span>" % action
        self.action.set_markup( tit)
        self.pkg = self.filterTree.get_widget( "confPkg" )
        self.pkgModel = self.setup_view(self.pkg)
        self.show_data(self.pkgModel, pkg)
        self.dep = self.filterTree.get_widget( "confDep" )        
        if len(dep) == 0:
            # Hide depencies frame
            depframe = self.filterTree.get_widget( "depFrame" )
            depframe.hide()
        else:
            self.depModel = self.setup_view(self.dep)
            self.show_data(self.depModel, dep)
        self.size = self.filterTree.get_widget( "confSize" )
        if action == "Remove":
            sizeframe = self.filterTree.get_widget( "sizeFrame" )
            sizeframe.hide()
        else:
            self.size.set_text( size )
        self.bOK = self.filterTree.get_widget( "confOK" )
        self.bOK.connect( "clicked", self.click, "OK" )
        self.bCancel = self.filterTree.get_widget( "confCancel" )
        self.bCancel.connect( "clicked", self.click, "CANCEL" )
        self.ret = False
        self.win.show() 
             
    def setup_view(self,view):
        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        view.set_model( model )
        self.create_text_column("Name",view,0,200)
        self.create_text_column("Ver",view,1,80)
        self.create_text_column("Size",view,2)
        return model

    def create_text_column( self, hdr, view, colno,min_width=0 ):
         cell = gtk.CellRendererText()    # Size Column
         column = gtk.TreeViewColumn( hdr, cell, text=colno )
         column.set_resizable( True )
         if not min_width == 0:
             column.set_min_width(min_width)
         view.append_column( column )        
             
             
    def show_data(self,model,pkg):
        model.clear()
        for name,ver,size in pkg:
            iterator = model.append()           
            model.set_value( iterator, 0, name )
            model.set_value( iterator, 1, ver )
            model.set_value( iterator, 2, size )            
            
        
    def quit( self, w=None, event=None ):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()

    def click( self, button, key ):
        if key == "OK":
            self.ret = True
        self.quit()

def confimation_dialog( xml, action, pkg, dep, size ):
    dlg = ConfimationDialog( xml, action, pkg, dep, size )
    gtk.main()
    return dlg.ret

class AboutDialog:
    def __init__( self, xmlfile, version ):
        self.filterTree=gtk.glade.XML ( xmlfile, 'about' )
        self.win = self.filterTree.get_widget( "about" )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.bOK = self.filterTree.get_widget( "aboutOK" )
        self.bOK.connect( "clicked", self.click, "OK" )
        self.image = self.filterTree.get_widget( "aboutImage" )
        path = os.path.abspath( os.path.dirname( xmlfile ) )
        self.image.set_from_file("/usr/share/pixmaps/yumex/logo.png" )
        self.image.show()
        self.ver = self.filterTree.get_widget( "aboutVer" )
        self.ver.set_text( version )
        self.win.show() 
                     
    def quit( self, w=None, event=None ):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()

    def click( self, button, key ):
        if key == "OK":
            self.ret = True
        else:
            self.ret = False
        self.quit()

def about_dialog( xml, version ):
    dlg = AboutDialog( xml, version )
    gtk.main()
    return dlg.ret

class ExcludeDialog:
    def __init__( self, xmlfile ):
        self.filterTree=gtk.glade.XML ( xmlfile, 'exclude' )
        self.win = self.filterTree.get_widget( "exclude" )
        self.win.connect( "destroy", self.quit )
        self.win.connect( "delete_event", self.quit )
        self.bOK = self.filterTree.get_widget( "excOK" )
        self.bOK.connect( "clicked", self.click, "OK" )
        self.bOK = self.filterTree.get_widget( "excCANCEL" )
        self.bOK.connect( "clicked", self.click, "CANCEL" )
        self.excList = self.filterTree.get_widget( "excView" )
        self.buffer = self.excList.get_buffer()
        self.settings = Settings( '/etc/yumex.conf', "excludes" )
        self.get_excludes()
        self.win.show() 
        
    def get_excludes( self ):
        ne = self.settings.get_value( "numexcludes" )
        if ne != None:
            num = int( ne )
            for i in range( 1, num+1 ):
                ne = self.settings.get_value( "exclude"+str( i ) )
                if ne != None:
                    self.addline( ne )
                    
    def save_excludes( self ):
        lines = self.get_lines()
        #print lines
        num = len( lines )
        if num > 0:
            self.settings.remove_section_elem()
            self.settings.set_value( "numexcludes", str( num ) )
            i = 1
            for lin in lines:
                if lin != "":
                    key = "exclude%i" % i
                    self.settings.set_value( key, lin )
                    i += 1
            self.settings.set_value( "numexcludes", str( i-1 ) )
            self.settings.save()
    
        
    def get_lines( self ):
        start = self.buffer.get_start_iter()
        end = self.buffer.get_end_iter()
        txt = self.buffer.get_text( start, end )
        return txt.split( '\n' )
        
                
    def addline( self, txt ):
        end = self.buffer.get_end_iter()
        self.buffer.insert( end, "%s\n" % txt )
                     
    def quit( self, w=None, event=None ):
        self.ret = False
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()

    def click( self, button, key ):
        if key == "OK":
            self.save_excludes()
            self.ret = True
        else:
            self.ret = False
        self.quit()

def exclude_dialog( xml ):
    dlg = ExcludeDialog( xml )
    gtk.main()
    return dlg.ret
        
        