#!/usr/bin/python -tt
#
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: output.py,v 1.3 2005/08/03 13:27:41 timlau Exp $

""" Yum Extender Output Classes """

import os
import sys
import locale
import time # test purposes only
import pygtk
import gtk
import pango
import re
import rpm

sys.path.insert( 0, '/usr/share/yum-cli' )

import yum
import output
import yum.Errors as Errors
import cli
import callback

from yum.packages import parsePackages, YumLocalPackage, YumInstalledPackage
from yum.logger import Logger

from urlgrabber.progress import *



from i18n import _

#from YumexUtils import *



class TextOutputStream:
    """
    A output stream wrapper
    """
    def __init__( self, old_out ):
        self.out = old_out

    def isatty( self ):
        return self.out.isatty()

    def flush( self ):
        return self.out.flush()

    def write( self, text ):
        self.out.write( text )
        

    def fileno( self ):
        return self.out.fileno()

class TextInputStream:
    """
    A input stream wrapper
    """
    def __init__( self, old_in, handler ):
        self.inp = old_in
        self.handler = handler

    def isatty( self ):
        return self.inp.isatty()

    def flush( self ):
        return self.inp.flush()

    def write( self, text ):
        self.inp.write( text )
        
    def read( self ):
        rc = self.handler.read()
        return rc
        
    def readline( self ):
        rc = self.handler.readline()
        return rc

    def fileno( self ):
        return self.inp.fileno()


class TextViewOutputStream:
    """
    A output stream wrapper
    """
    def __init__( self, old_out, handler, style, echo = False ):
        self.out = old_out
        self.handler = handler
        self.style = style
        self.echo = echo

    def isatty( self ):
        return True

    def flush( self ):
        return self.out.flush()

    def write( self, text ):
        if self.echo:
            self.out.write( text )
        self.handler.write( text, self.style )
        

    def fileno( self ):
        return self.out.fileno()

class NoteBookConsole:
    """
     GTK Console with redirected stdout & stderr.
    """
    def __init__( self, sw, textview, debug ):
 
        self.scroll = sw
        self.scroll.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS )
        self.text = textview
        self.text.set_wrap_mode( True )
        self.text.set_editable( False )
        self.hidden = True
        self.debug = debug
        self.crt = 0
        self.pattern = re.compile( '\d+[/]\d+' )
      


        self.mark = self.text.get_buffer().create_mark( "End", self.text.get_buffer().get_end_iter(), False )

        #setup styles.
        self.style_banner = gtk.TextTag( "banner" )
        self.style_banner.set_property( "foreground", "saddle brown" )
        self.style_banner.set_property( "family", "Monospace" )
        self.style_banner.set_property( "size_points", 7 )

        self.style_ps1 = gtk.TextTag( "ps1" )
        self.style_ps1.set_property( "foreground", "DarkOrchid4" )
        self.style_ps1.set_property( "editable", False )
        self.style_ps1.set_property( "family", "Monospace" )
        self.style_ps1.set_property( "size_points", 8 )

        self.style_ps2 = gtk.TextTag( "ps2" )
        self.style_ps2.set_property( "foreground", "DarkOliveGreen" )
        self.style_ps2.set_property( "editable", False )
        self.style_ps2.set_property( "font", "courier" )

        self.style_out = gtk.TextTag( "stdout" )
        self.style_out.set_property( "foreground", "midnight blue" )
        self.style_out.set_property( "family", "Monospace" )
        self.style_out.set_property( "size_points", 7 )


        self.style_err = gtk.TextTag( "stderr" ) 
        self.style_err.set_property( "style", pango.STYLE_ITALIC )
        self.style_err.set_property( "foreground", "red" )
        self.style_err.set_property( "family", "Monospace" )
        self.style_err.set_property( "size_points", 8 )

        self.text.get_buffer().get_tag_table().add( self.style_banner )
        self.text.get_buffer().get_tag_table().add( self.style_ps1 )
        self.text.get_buffer().get_tag_table().add( self.style_ps2 )
        self.text.get_buffer().get_tag_table().add( self.style_out )
        self.text.get_buffer().get_tag_table().add( self.style_err )
        
        #Create output streams for stdout & stderr into GTK Textview 
        self.stdout = TextViewOutputStream( sys.stdout, self, self.style_out, self.debug )
        self.stderr = TextViewOutputStream( sys.stderr, self, self.style_err, True )
        self.stdin = TextInputStream( sys.stdin, self )
        sys.stderr = self.stderr
        sys.stdout = self.stdout
        sys.stdin = self.stdin

    def write_line( self, txt, style=None ):
        start, end = self.text.get_buffer().get_bounds()
        if style == None:
            self.text.get_buffer().insert_with_tags( end, txt, self.style_out )
        else:
            self.text.get_buffer().insert_with_tags( end, txt, style )
        self.text.scroll_to_mark( self.text.get_buffer().get_insert(), 0 )

    def current_line( self ):
        start, end = self.current_line_bounds()
        return self.text.get_buffer().get_text( start, end, True )

    def current_line_bounds( self ):
        txt_buffer = self.text.get_buffer()
        l = txt_buffer.get_line_count() - 1
        start = txt_buffer.get_iter_at_line( l )
        end = txt_buffer.get_end_iter()
        return start, end
        
    def scroll_to_bottom( self ):
        txt_buffer = self.text.get_buffer()
        self.text.scroll_to_mark( txt_buffer.get_insert(), 0 )
        

    def delete_line( self ):
        start, end = self.current_line_bounds()
        self.text.get_buffer().delete( start, end )

    def replace_line( self, txt, style=None ):
        start, end = self.current_line_bounds()
        self.text.get_buffer().delete( start, end )
        if style == None:
            self.text.get_buffer().insert_with_tags( start, txt, self.style_out )
        else:
            self.text.get_buffer().insert_with_tags( start, txt, style )
        self.text.scroll_to_mark( self.text.get_buffer().get_insert(), 0 )
            

    def clear( self ):
        self.text.get_buffer().set_text( "" )

    def write( self, text, style=None ):
        if len( text ) != 0:
            if text[0] == ( '\r' ):
                text = text[1:]
                self.replace_line( text, style )
            else:
                self.write_line( text, style )
        else:
            self.write_line( text, style )
        while gtk.events_pending():      # process gtk events
            gtk.main_iteration()
    
    def get_count( self, txt ):
        result = self.pattern.search( txt )
        if result != None:
            #print result
            values = result.group( 0 ).split( '/' )
            c1 = int( values[0] )
            c2 = int( values[1] )
            return c1, c2
        else:
            return 0, 0
    
    def readline( self ):
        start, end = self.current_line_bounds()
        txt = self.text.get_buffer().get_text( start, end )  
        rc = warning_box( "Input", txt )
        if rc == "OK":
          print "y" 
          return "y\n"
        else:
          print "n" 
          return "n\n"

