#!/usr/bin/python -tt
#
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: utils.py,v 1.11.2.2 2005/10/13 13:10:04 timlau Exp $

""" Yum Extender Utility Classes """
 
import sys, os, gobject, glob, shutil, datetime
import gtk
import gtk.glade
import ConfigParser
import pygtk

from types import *
from i18n import _

#from yumbase import YumWrapper
import dialogs 


class grpInfoView:
    """ Group Info View Class"""
    def __init__( self,widget):
        self.view = widget
        self.model = self.setup_view()

    def setup_view( self ):
        """ Create Notebook list for single page  """
        model = gtk.TreeStore( gobject.TYPE_STRING)           
        self.view.set_model( model )
        
        cell1 = gtk.CellRendererText()
        column1= gtk.TreeViewColumn( "Group", cell1,text=0 )
        column1.set_resizable( True )
        column1.set_min_width(450)
        self.view.append_column( column1 )       
        return model


    def populate_view( self, data):
        """ Populate a Notebook page with data """
        self.model.clear()
        for grp,lvl0 in data:
            for sub,lvl1 in lvl0:
                level1 = self.model.append(None,sub)
                for pkg in lvl2:
                    level2 = self.model.append(level1,pkg)
            


class grpView:
    """ Group View Class"""
    def __init__( self,widget):
        self.view = widget
        self.model = self.setup_view()
    
    def on_toggled( self, cell, path, user_data):
        model, column = user_data
        #print path
        model[path][column] = not model[path][column]
        return

    def setup_view( self ):
        """ Create Notebook list for single page  """
        model = gtk.TreeStore( gobject.TYPE_STRING, 'gboolean')           
        self.view.set_model( model )
        
        cell1 = gtk.CellRendererText()
        column1= gtk.TreeViewColumn( "Group", cell1,text=0 )
        column1.set_resizable( True )
        column1.set_min_width(450)
        self.view.append_column( column1 )
        
        cell2 = gtk.CellRendererToggle()    # Selection
        cell2.set_property( 'activatable', True )
        column2 = gtk.TreeViewColumn( " ", cell2 )
        column2.add_attribute( cell2, "active", 1 )
        column2.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column2.set_fixed_width(20)
        column2.set_resizable( False )
        self.view.append_column( column2 )
        cell2.connect( "toggled", self.on_toggled, ( model, 1 ) )
        return model


    def populate_view( self, data):
        """ Populate a Notebook page with data """
        self.model.clear()
        for state,lvl0 in data:
            root = self.model.append(None,(state,None))
            for grp in lvl0:
                level0 = self.model.append(root,(grp,None))
#                for sub,lvl2 in lvl1:
#                    level1 = self.model.append(level0,(sub,None))
#                    for pkg in lvl2:
#                        level2 = self.model.append(level1,(pkg,None))
#            
            
class pkgView:
    """ Package and repo View Class"""
    def __init__( self,widget, headers=None, handler=None, new_handler=None ):
        self.view = widget
        self.handler = handler
        self.new_handler = new_handler
        if headers == None:
            self.headers = [_( "Package" ), _( "Ver" ), _( "Summary" ), _( "Repo" ), _( "Architecture" ), _( "Size" )]
            self.newIndicator = True
        else:    
            self.headers = headers
            self.newIndicator = False            
        self.model = self.setup_view()
        self.columnClickState = False
    
    def on_toggled( self, widget, column, model ):
        """ Notebook list checkbox handler """
        col = model[0]
        row = col[int( column )]
        row[0] = not row[0]
        return

    def on_column_click( self, col ): 
        if self.columnClickState:
            self.select_all()
            self.columnClickState = False
        else:
            self.deselect_all()
            self.columnClickState = True
        
    def setup_view( self ):
        """ Create Notebook list for single page  """
        model = gtk.ListStore( 'gboolean',gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, 'gboolean')
            
        self.view.set_model( model )
        cell1 = gtk.CellRendererToggle()    # Selection
        cell1.set_property( 'activatable', True )
        column1 = gtk.TreeViewColumn( "    ", cell1 )
        column1.add_attribute( cell1, "active", 0 )
        column1.set_resizable( True )
        self.view.append_column( column1 )
        if self.handler == None:
            cell1.connect( "toggled", self.on_toggled, ( model, column1 ) )
        else:
            cell1.connect( "toggled", self.handler, ( model, column1 ) )
            
        column1.set_clickable( True )
        column1.connect( "clicked", self.on_column_click ) 

        if self.newIndicator:
            cell2 = gtk.CellRendererPixbuf()    # new
            cell2.set_property( 'stock-id', gtk.STOCK_ADD)
            column2 = gtk.TreeViewColumn( "    ", cell2)
            column2.set_cell_data_func(cell2, self.new_pixbuf)
            column2.set_resizable( True )
            self.view.append_column( column2 )
            if not self.new_handler is None:
                column2.set_clickable( True )
                column2.connect( "clicked", self.new_handler ) 
        
        for i in [1,2,3,4,5,6]:       
            self.create_text_column( self.headers[i-1],self.view, i )
        return model

    def create_text_column( self, hdr, view, colno ):
         cell = gtk.CellRendererText()    # Size Column
         column = gtk.TreeViewColumn( hdr, cell, text=colno )
         column.set_resizable( True )
         if hdr == "":
              column.set_visible( False )
         view.append_column( column )        
         
    def new_pixbuf(self, column, cell, model, iter):
        isNew = model.get_value(iter, 7)
        cell.set_property('visible', isNew)

    def populate_view( self, data ):
        """ Populate a Notebook page with data """
        self.model.clear()
        for col0, col1, col2, col3, col4, col5, col6, col7 in data:
            iterator = self.model.append()           
            self.model.set_value( iterator, 7, col7 )
            self.model.set_value( iterator, 6, col6 )
            self.model.set_value( iterator, 5, col5 )
            self.model.set_value( iterator, 4, col4 )
            self.model.set_value( iterator, 3, col3 )
            self.model.set_value( iterator, 2, col2 )
            self.model.set_value( iterator, 1, col1 )
            self.model.set_value( iterator, 0, col0 )
            
    def get_selected(self,package=True):
        selected = []
        for elem in self.model:
            state = elem[0]
            name = elem[1]
            arch = elem[5]
            fullver = elem[2]
            if package:
                fe = fullver.find( ":" )
                if fe > -1:
                    epoch = fullver[0:fe]
                    ver = fullver[fe+1:]
                    add = "%s:%s-%s.%s" % ( epoch, name, ver, arch )
                else:
                    add = "%s:%s-%s.%s" % ( '0', name, fullver, arch )
                #add = "%s.%s" % (name,arch)
            else:
                add = name
            if state:
                selected = selected + [add]
        return selected
        
    def get_notselected( self):
        notselected = []
        for elem in self.model:
            state = elem[0]
            name = elem[1]
            if not state:
                notselected = notselected + [name]
        #print selected
        return notselected
            
    def deselect_all( self):
        iterator = self.model.get_iter_first()
        while iterator != None:    
            self.model.set_value( iterator, 0, False )
            iterator = self.model.iter_next( iterator )

    def select_all( self):
        iterator = self.model.get_iter_first()
        while iterator != None:    
            self.model.set_value( iterator, 0, True )
            iterator = self.model.iter_next( iterator )
            

    def select_by_keys( self, keys,isPkgs=True ):
        model = self.model
        iterator = model.get_iter_first()
        while iterator != None:    
            if isPkgs:
                pkg = self.get_fullpkg( iterator )
            else:
                pkg = model.get_value( iterator, 1 )
            if pkg in keys:
                model.set_value( iterator, 0, True )
            else:
                model.set_value( iterator, 0, False )
            iterator = model.iter_next( iterator )
  
    def get_fullpkg( self, iterator ):
        name = self.model.get_value( iterator, 1 )
        arch = self.model.get_value( iterator, 5 )
        fullver = self.model.get_value( iterator, 2 )
        fe = fullver.find( ":" )
        if fe > -1:
            epoch = fullver[0:fe]
            ver = fullver[fe+1:]
            pkg = "%s:%s-%s.%s" % ( epoch, name, ver, arch )
        else:
            pkg = "%s:%s-%s.%s" % ( '0', name, fullver, arch )
        return pkg

class Settings:
    """ This class is used to read and write configuration from
        /etc/yumex.conf """
        
    def __init__( self, cfgfile='/etc/yumex.conf', sec='yumex' ):
        self.conf_file = cfgfile
        self.section = sec
        self.conf = ConfigParser.ConfigParser()
        self.conf.read( self.conf_file )             
        if not self.conf.has_section( self.section ):
            self.conf.add_section( self.section )
            

    def get_option( self, option ):
        if self.conf.has_option( self.section, option ):
            return self.conf.get( self.section, option )
        else:
            return None
        

    def is_enabled( self, option ):
        if self.conf.has_option( self.section, option ):
            if self.conf.get( self.section, option ) == "1":
                return True
            else:
                return False
        else:
            return False

    def load( self ):
        self.conf.read( self.conf_file )             
        
    def get_value( self, option ):
        if self.conf.has_option( self.section, option ):
            return self.conf.get( self.section, option )
        else:
            return None

        
    def set_value( self, option, value="" ):
        if value != "":
            self.conf.set( self.section, option, value )
        else:
            if self.conf.has_option( self.section, option ):
                self.conf.remove_option( self.section, option )
    
    def set_boolean( self, option, state ):
        if state:
            self.conf.set( self.section, option, "1" )
        else:
            self.conf.set( self.section, option, "0" )

    def get_boolean( self, option ):
        if self.conf.has_option( self.section, option ):
            if self.conf.get( self.section, option ) == "1":
                return True
            else:
                return False
        else:
            return False
        
    def save( self ):
        f=open( self.conf_file, "w" )
        self.conf.write( f )
        f.close()        
        
    def remove_section_elem( self ):
        if self.conf.has_section( self.section ):
            items = self.conf.options( self.section )
            for i in items:
                self.conf.remove_option( self.section, i )
                
            
    def add_section( self ):
        if not self.conf.has_section( self.section ):
            self.conf.add_section( self.section )
        


class Profile:
    """ Class for handling repo selection Profile """
    def __init__( self ):
        self.profiles = ConfigParser.ConfigParser()
        self.filename = '/etc/yumex.profiles.conf'
        self.profiles.read( self.filename )
        self.active=self.profiles.get( "main", "LastProfile" )    
        self.proDict = {}    
        self.load()        

    
    def save( self ):
        f=open( self.filename, "w" )
        self.profiles.write( f )
        f.close()
        
    def load( self ):
        self.profiles.read( self.filename )
        self.active=self.profiles.get( "main", "LastProfile" )        
        for p in self.profiles.sections():
            if p != 'main':
                opts = self.profiles.options( p )
                self.proDict[p] = opts
        #print self.proDict

    def get_profile( self, name ):
        if name in self.proDict.keys():
            return self.proDict[name]
        else:
            return False

    def set_profile( self, name, repos ):
        if self.profiles.has_section( name ):
            self.profiles.remove_section( name )    
            self.add_profile( name, repos )                
            return True
        else:
            return False
        

    def get_list( self ):
        return self.proDict.keys()

    def get_active( self ):
        return self.active

    def set_active( self, name ):
        if self.profiles.has_section( name ):
            self.active = name
            self.profiles.set( "main", "LastProfile", name )        
            self.save()

    def add_profile( self, name, repos ):
        if not self.profiles.has_section( name ):
            self.profiles.add_section( name )
            for r in repos:
                self.profiles.set( name, r, "1" )
            self.save()
            self.load()
            return True
        else:
            return False
            

class Repository:
    """ Class for handling .repo files """
    def __init__( self ):
        self.baseRepos = ['updates-released', 'base', 'development', 'updates-testing']
        self.cfg = ConfigParser.ConfigParser()
        self.update()
        #self.list_all()
    
    def list_all( self ):                                    # List all repos.
        for r in self.repos:
            self.set_active( r )
            print "[%s]" % r
            opt = self.list_options()
            for o in opt:
                if o in ["enabled", "gpgcheck"]:
                    bu = self.get_flag( o )
                    print o, " : ", bu
                    print "%s=%s" % ( o, self.get_option( o ) )
                else:
                    print "%s=%s" % ( o, self.get_option( o ) )
            
    def list_active( self ):                            # List active repo
        print "[%s]" % self.activeRepo
        print "%s=%s" % ( "name", self.get_option( "name" ) )
        if self.isBaseUrl():
            print "%s=%s" % ( "baseurl", self.get_option( "baseurl" ) )
        else:
            print "%s=%s" % ( "mirrorlist", self.get_option( "mirrorlist" ) )
        opt = self.list_options()
        for o in opt:                                            
            if o in ['name', 'baseurl', 'mirrorlist']:    
                pass
            else:
                value = self.get_option( o )                    
                print "%s=%s" % ( o, value )
        
                                            
    def load_repos( self ):                    # Load repo information from repofiles
        files = self.fileDict.keys()        # Load repo info for files in file dict.
        for f in files:
            self.cfg.read( f )
        if self.cfg.has_section( 'main' ):    # Remove yum.conf [main] if exist
            self.yum_conf_main = self.cfg.items( 'main' )
            self.cfg.remove_section( 'main' )
        
    def build_section_dict( self ):            # build repo section, repo file directory
        tcfg = ConfigParser.ConfigParser()
        r = '/etc/yum.conf'
        tcfg.read( r )
        for s in tcfg.sections():            # Parse sections
            #print "    Section : "+s
            if s != "main":
                self.repoDict[s] = r
                if r in self.fileDict.keys():
                    self.fileDict[r] =  self.fileDict[r]+[s]
                else:
                    self.fileDict[r] = [s]
        if len( tcfg.sections() ) > 1: 
            ycRepos = self.fileDict[r]
        else:
            ycRepos =[]
        #print ycRepos
        for r in self.repofiles:                # Parse .repo files
            tcfg = ConfigParser.ConfigParser()
            tcfg.read( r )
            #print "File : "+r    
            for s in tcfg.sections():            # Parse sections
                #print "    Section : "+s
                if s in ycRepos:                    # Check if repos allready definded in yum.conf
                    shutil.copy( r, r+".yumex" )        # Rename .repo file to .repo.yumex
                    os.remove( r )                        # Remove file
                else:
                    self.repoDict[s] = r
                    if r in self.fileDict.keys():
                        self.fileDict[r] = self.fileDict[r] + [s]
                    else:
                        self.fileDict[r] = [s]
        #print self.repoDict
        #print self.fileDict
        
    def update( self ):                            # Updates class member, reloads repos
        
        self.repofiles = glob.glob( '/etc/yum.repos.d/*.repo' )
        #self.repofiles = glob.glob('repos/*.repo')
        self.repofiles.sort()
        self.repoDict =  {}
        self.fileDict = {}
        self.yum_conf_main = []
        self.build_section_dict()
        self.load_repos()
        self.repos = self.read_names()
        self.filename = ""
        self.activeRepo = ""
        
    def isBaseUrl( self ): # If repo has baseurl=
        return self.cfg.has_option( self.activeRepo, "baseurl" )
            
    def read_names( self ):    # return all repo titles.
        return self.cfg.sections()
    
    def set_active( self, repo ):    # set active repo
        self.activeRepo = repo
    
    def get_flag( self, option, default=False ):    # read flag from active repo
        try:
            return self.cfg.getboolean( self.activeRepo, option )
        except ( ConfigParser.NoSectionError, ConfigParser.NoOptionError ), e:                
            return default

    def list_options( self ):    # returns all options from active repo
        return self.cfg.options( self.activeRepo )
    
    def get_option( self, option, default=None ):    # read option value from active repo
        try:
            return self.cfg.get( self.activeRepo, option )
        except ( ConfigParser.NoSectionError, ConfigParser.NoOptionError ), e:
            return default

    def set_flag( self, option, state ):                # Set flag in active Repo
        if state:
            return self.cfg.set( self.activeRepo, option, "1" )
        else:
            return self.cfg.set( self.activeRepo, option, "0" )
        
    
    def set_option( self, option, value ):                # Set option in active repo
        return self.cfg.set( self.activeRepo, option, value )

    def remove_option( self, option ):                    # remove option in active repo
        return self.cfg.remove_option( self.activeRepo, option )
    
    def set_filename( self, key ):                        # set filename from title lookup
        self.filename = self.repoDict[key]
        #print self.filename
        
    def get_repo_filename( self, key ):
        return self.repoDict[key]        

    def get_names_sorted( self ):    # Returns sort list of repos (base repos first)
        blist = []
        olist = []
        repos = self.read_names()
        for r in repos:        
            if r in self.baseRepos:
                blist.append( r )
        blist.sort()
        for r in repos:
            if r in self.baseRepos:
                pass
            else:
                olist.append( r )
        olist.sort()    
        blist.extend( olist )    # Append otherlist in end of baselist
        return blist
        
    def save( self ):        # Save active repo to .repo file.
        sec = self.activeRepo
        self.filename = self.repoDict[sec]
        shutil.copy( self.filename, self.filename+".bak" )    # Create Backup
        self.save_repo_file()
        self.update()                                # Reload repos
        self.activeRepo = sec    

    def list_exclude( self, list1, list2 ):
        data = []
        for el in list1:
            if el not in list2:
                data.append( el )
        return data


    def delete_repo( self, name ):
        if name in self.repos:
            f = self.repoDict[name]             # File were name is stored
            repos = self.fileDict[f]            # All repos in this file
            self.filename = f
            if len( repos ) == 1:                # is name the only repo in file 
                os.remove( f )                   # Remove file
                self.cfg.remove_section( name ) 
                del self.fileDict[f]             # remove repo file from fileDirec     
            else:
                self.cfg.remove_section( name )                        
                self.fileDict[f] = self.list_exclude( repos, [name] ) # repos - name.
                self.save_repo_file()                
            self.update()  
             
                        
    def create_new( self, title, name=None, urltype="baseurl", url="", enable="0", gpg="0", gpgkey=None ):
        """ Create new Repository """
        if title in self.repos:
            sys.stderr.write( "Repository with name %s allready exist..\n" % title )
            return False
        self.filename = "/etc/yum.repos.d/"+title+".repo"
        self.cfg.add_section( title )
        if name == None:
            name = title
        self.cfg.set( title, "name", name )
        if urltype == "baseurl":
            self.cfg.set( title, "baseurl", url )
        else:
            self.cfg.set( title, "mirrorlist", url )
        self.cfg.set( title, "enabled", enable )
        self.cfg.set( title, "gpgcheck", gpg )
        if gpgkey != None:
            self.cfg.set( title, "gpgkey", gpgkey )
        self.activeRepo = title 
        self.fileDict[self.filename] = [title]       
        self.save_repo_file()
        self.update()                                # Reload repos
        self.activeRepo = title                
        #self.list_active()
        return True

    def write_repo_to_file( self, f ):
        f.write( "#\n" ) 
        f.write( "# Repository : %s \n" % self.activeRepo )
        f.write( "#\n" ) 
        f.write( "[%s]\n" % self.activeRepo )
        f.write( "%s=%s\n" % ( "name", self.get_option( "name" ) ) )
        if self.isBaseUrl():
            urls = self.get_option( "baseurl" ).split( '\n' )
            #print urls
            first = True
            for url in urls:
                if first:
                    f.write( "%s=%s\n" % ( "baseurl", url ) )
                    first = False
                elif url.strip() != '':
                    f.write( "          %s\n" % ( url ) )                  
        else:
            f.write( "%s=%s\n" % ( "mirrorlist", self.get_option( "mirrorlist" ) ) )
        opt = self.list_options()
        for o in opt:                                            
            if o in ['name', 'baseurl', 'mirrorlist']:    
                pass
            else:
                value = self.get_option( o )                    
                f.write( "%s=%s\n" % ( o, value ) )
        f.write( "\n" ) 


    def save_repo_file( self ):
        #print "Saving to : "+self.filename  
        sections = self.fileDict[self.filename]
        #print sections        
        f=open( self.filename, "w" )
        if self.filename == '/etc/yum.conf':
            self.write_yum_main( f )
        today = datetime.date.today()
        f.write( "# Repositories updated : %s\n\n" % today )
        for s in sections:
            self.activeRepo = s
            self.write_repo_to_file( f )
        f.close()
    
    def write_yum_main( self, f ):
        f.write( "#\n" ) 
        f.write( "# Yum main Configuration\n" )
        f.write( "#\n" ) 
        f.write( "[main]\n" )
        for key, value in self.yum_conf_main:
            f.write( "%s=%s\n" % ( key, value ) )
        f.write( "\n# PUT YOUR REPOS HERE OR IN separate files named file.repo\n" )
        f.write( "# in /etc/yum.repos.d\n\n" )
    
        
