#!/usr/bin/python -tt
#
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: yumbase.py,v 1.12.2.8 2005/10/31 08:03:09 timlau Exp $

# Yum Extender Yum Classes & callback handlers

import os
import sys
import locale
import time # test purposes only
import pygtk
import gtk
import pango
import re

import rpm

sys.path.insert( 0, '/usr/share/yum-cli' )

import yum
import output
import yum.Errors as Errors
import cli
import callback
import rpmUtils
import fcntl
import fnmatch



from yum.packages import parsePackages, YumLocalPackage, YumInstalledPackage
from yum.logger import Logger
import urlgrabber

from urlgrabber.progress import *

from i18n import _

import dialogs
import utils

class yumexBase( cli.YumBaseCli):
    """ This class does all the interfacing with yum """

    def __init__( self, console=None, progress=None, loglevel=2,options=[]):
        cli.YumBaseCli.__init__( self )
        self.progress = progress
        self.console = console
        self.last_error = None
        self.updates = []
        self.installed = []
        self.available = []
        self.loglevel = loglevel
        self.options = options
        self.debugMode = False
        if 'debug' in self.options:
            self.debugMode = True

    def setup( self ):
        try:
            self.progresslog(_("Setup Yum : Config"))
            self.doConfigSetup()        
            self.setupFileLog()
            self.conf.getConfigOption('assumeyes',True) # Disable questions asked
            self.conf.setConfigOption('uid', os.geteuid()) # who are we
            self.setup_excludes()
            self.progresslog(_("Setup Yum : Plugins"))
            # Dont init plugins they, can mess up thing
            #self.doPluginSetup()
            self.progresslog(_("Setup Yum : Transaction Set"))
            self.repos.callback = CacheProgressCallback( self.log, self.errorlog, self.filelog, self.console, self.progress )
            self.doTsSetup()
            self.progresslog(_("Setup Yum : RPM Db."))
            self.doRpmDBSetup()
            self.progresslog(_("Setup Yum : Base setup completed"))
            return True
        except yum.Errors.RepoError, e:
            self.last_error = str( e )
            return False

    def setup_excludes( self ):
        set = utils.Settings( '/etc/yumex.conf', "excludes" )
        ne = set.get_value( "numexcludes" )
        if ne != None:
            num = int( ne )
            for i in range( 1, num+1 ):
                ne = set.get_value( "exclude"+str( i ) )
                if ne != None:
                    try:
                        excludelist = self.conf.getConfigOption('exclude')
                        excludelist.append(ne)
                        self.conf.setConfigOption('exclude', excludelist)
                    except yum.Errors.ConfigError, e:
                        self.errorlog(0, _(e))

        
    def load_repo_data( self ):
        self.progresslog(_("Loading Repositories Data"))
        self.repos.setProgressBar( MyTextMeter( self.console, self.progress ) )
        self.doTsSetup()
        try:
            self.doRepoSetup()
            #self.progresslog(_("Setup Package Sack"))
            self.doSackSetup() 
            if 'filelists' in self.options:
                self.repos.populateSack(with='all')            
            #self.progresslog(_("Loading Repository Group Data"))
            #self.doGroupSetup()
            self.progresslog(_("Loading Repositories Data completed"))
            return True
        except yum.Errors.RepoError, e:
            self.last_error = str( e )
            return False
    
    def doRepoSetup(self):
        """ Overloading the cli.YumBaseCli.doRepoSetup
            calls yum.YumBase.doRepoSetup directly """
        yum.YumBase.doRepoSetup(self) # Call Yumbase reposetup routine.
                
        
    def doGroupSetup(self):
        """ Get Group Info from repos"""
        try:
            yum.YumBase.doGroupSetup(self)
        except yum.Errors.GroupsError, e:
            self.log(2,str(e))
            
    def userconfirm(self):
        return True
        
    def getGroupListData(self):
        inst,avail = self.doGroupLists()
        data = []
        grps = []
        for grp in inst:
            grps.append(grp)
        data.append(("Installed Groups",grps))
        grps = []
        for grp in avail:
            grps.append(grp)
        data.append(("Avaliable Groups",grps))        
        return data

    def getGroupData(self, group):
        #print '\nGroup: %s' % group
        main = []
        groupid = self.groupInfo.matchGroup(group)
        if len(self.groupInfo.sub_groups[groupid]) > 0:
            sub = 'Required Groups'
            pkgs = []
            for id in self.groupInfo.sub_groups[groupid]:
                grp = self.groupInfo.group_by_id[id]
                pkgs.append(grp.name)
            main.append([sub,pkgs])
        if len(self.groupInfo.default_metapkgs[groupid]) > 0:
            sub = 'Default Metapkgs'
            pkgs = []
            for id in self.groupInfo.default_metapkgs[groupid]:
                grp = self.groupInfo.group_by_id[id]
                pkgs.append(grp.name)
            main.append([sub,pkgs])
        if len(self.groupInfo.optional_metapkgs[groupid]) > 0:
            sub = 'Optional Metapkgs'
            pkgs = []
            for id in self.groupInfo.optional_metapkgs[groupid]:
                grp = self.groupInfo.group_by_id[id]
                pkgs.append(grp.name)
            main.append([sub,pkgs])
        if len(self.groupInfo.mandatory_pkgs[groupid]) > 0:
            sub = 'Mandatory Packages'
            pkgs = []
            for item in self.groupInfo.mandatory_pkgs[groupid]:
                pkgs.append(item)
            main.append([sub,pkgs])
        if len(self.groupInfo.default_pkgs[groupid]) > 0:
            sub = 'Default Packages'
            pkgs = []
            for item in self.groupInfo.default_pkgs[groupid]:
                pkgs.append(item)
            main.append([sub,pkgs])
        if len(self.groupInfo.optional_pkgs[groupid]) > 0:
            sub = 'Optional Packages'
            pkgs = []
            for item in self.groupInfo.optional_pkgs[groupid]:
                pkgs.append(item)
            main.append([sub,pkgs])
        return [group,main]
        
    def set_active_repos( self, repolist ):
        self.progresslog(_("Enable/Disable Repositories"))
        try:
            if repolist != None:
                for repo in self.repos.repos.values():
                    if repo.id not in repolist:
                        self.repos.disableRepo( repo.id )                                               
                        #print "Disabled : %s " % repo.id
                    else:
                        self.repos.enableRepo( repo.id )                                               
                        #print "Enabled  : %s " % repo.id
        except yum.Errors.RepoError, e:
            self.last_error = str( e )
            return False
        return True
        
    def build_package_lists( self ):
        self.progresslog(_("Building Package Lists : Updates"))       
        ypl = self.doPackageLists( pkgnarrow='updates' ) # Get only updates
        self.updates = ypl.updates
        self.progresslog(_("Building Package Lists : Installed"))       
        ypl = self.doPackageLists( pkgnarrow='installed' ) # Get only installed
        self.installed = ypl.installed
        self.progresslog(_("Building Package Lists : Available"))       
        ypl = self.doPackageLists( pkgnarrow='available' ) # Get only available
        self.available = ypl.available
        self.progresslog(_("Building Package Lists completed"))       
        
    def set_state( self, state ):
        self.progresslog( state )
        
    def errorlog( self, value, msg ):
        msg = str(msg)
        self.console.write_line( msg+"\n", self.console.style_err )
        self.doGtkEvents()


    def log( self, value, msg ):
        #print msg
        msg = str(msg)
        if self.debugMode:
            now = time.strftime("%H:%M:%S",time.localtime())
            msg = now +" : "+msg
            #print msg
        if value <= self.loglevel:
            self.console.write_line( msg+"\n", self.console.style_ps1 )
        self.doGtkEvents()
        
#    def filelog( self, value, msg ):
#        if value <= self.loglevel:
#            self.console.write_line( "filelog: "+msg+"\n", self.console.style_ps1 )
#        self.doGtkEvents()

    def progresslog(self, msg , writeLog=False):
        self.progress.set_subLabel(msg)
        self.progress.set_progress(0," ") # Blank the progress bar.
        self.doGtkEvents()
        if writeLog or self.debugMode:
            self.log(2,msg)

        
    def install( self, pkglist ):
        self.progresslog( _( "Setting up Install Process" ),True )        
        try:
            rc,msg = self.installPkgs(pkglist)
            if rc == 2:
                return self.buildTransaction() 
            else:
                return rc,msg
        except yum.Errors.YumBaseError, e:
            return 1, [str(e)]
        
    def update( self, pkglist ):
        self.progresslog( _( "Setting up Update Process" ),True )
        try:
            rc,msg = self.updatePkgs(pkglist)
            if rc == 2:
                return self.buildTransaction() 
            else:
                return rc,msg
        except yum.Errors.YumBaseError, e:
            return 1, [str(e)]

    def remove( self, pkglist ):
        self.progresslog( _( "Setting up Remove Process" ),True )
        try:
            rc,msg = self.erasePkgs(pkglist)
            if rc == 2:
                return self.buildTransaction() 
            else:
                return rc,msg
        
        except yum.Errors.YumBaseError, e:
            return 1, [str(e)]
        
        
    def process_transaction( self ):
        try:
            self.progress.show()
            self.download_packages()
            self.transaction_test()
            self.run_transaction()
            self.progress.hide()
            return True
        except Errors.YumBaseError, e:
            self.last_error = str( e )
            return False
            
    def download_packages( self ):
        """ Downloading packages and check GPG Signature """
        self.progresslog( _( 'Downloading Packages:' ), True )
        downloadpkgs = []
        for txmbr in self.tsInfo.getMembers():
            if txmbr.ts_state in ['i', 'u']:
                po = txmbr.po
                if po:
                    downloadpkgs.append( po )
    
        #self.log( 2, 'Downloading Packages:' )
         
        problems = self.downloadPkgs( downloadpkgs ) 
    
        if len( problems.keys() ) > 0:
            errstring = ''
            errstring += 'Error Downloading Packages:\n'
            for key in problems.keys():
                errors = yum.misc.unique( problems[key] )
                for error in errors:
                    errstring += '  %s: %s\n' % ( key, error )
                raise yum.Errors.YumBaseError(errstring)
                  # Check GPG signatures
        self.progresslog( _('Checking GPG Signatures:'),True )
        self.gpgsigcheck( downloadpkgs )

    def transaction_test(self):
        """ Do Traaction Test"""
        #self.log( 2, 'Running Transaction Test' )
        self.progresslog( _( 'Running Transaction Test' ),True )
        tsConf = {}
        for feature in ['diskspacecheck']: # more to come, I'm sure
            tsConf[feature] = self.conf.getConfigOption( feature )

        testcb = myRPMInstallCallback( 0, self.console, self.progress )
        testcb.tsInfo = self.tsInfo
        # clean out the ts b/c we have to give it new paths to the rpms 
        del self.ts
  
        self.initActionTs()
        # save our dsCallback out
        dscb = self.dsCallback
        self.dsCallback = None # dumb, dumb dumb dumb!
        self.populateTs( keepold=0 ) # sigh
        tserrors = self.ts.test( testcb, conf=tsConf )
        del testcb
  
        #self.log( 2, 'Finished Transaction Test' )
        self.progresslog( _( 'Finished Transaction Test' ),True )
        if len( tserrors ) > 0:
            errstring = 'Transaction Check Error: '
            for descr in tserrors:
                 errstring += '  %s\n' % descr 
            raise yum.Errors.YumBaseError(errstring) 

        #self.log( 2, 'Transaction Test Succeeded' )
        self.progresslog( _( 'Transaction Test Succeeded' ),True )
        del self.ts
        # put back our depcheck callback
        self.dsCallback = dscb
        

    def run_transaction(self):
        self.initActionTs() # make a new, blank ts to populate
        self.populateTs( keepold=0 ) # populate the ts
        self.ts.check() #required for ordering
        self.ts.order() # order
        output = 1
        if self.conf.debuglevel < 2:
            output = 0
        cb = myRPMInstallCallback( output, self.console, self.progress )
        cb.filelog = self.filelog # needed for log file output
        cb.tsInfo = self.tsInfo

        #self.log( 2, 'Running Transaction' )
        self.progresslog( _( 'Running Transaction' ),True )
        self.runTransaction( cb=cb )

        # close things
        self.log( 1, self.postTransactionOutput() )
      
      
    def setupFileLog(self):  
        rc = True
        # syslog-style log
        if self.conf.getConfigOption('uid') == 0:
            logpath = os.path.dirname(self.conf.logfile)
            if not os.path.exists(logpath):
                try:
                    os.makedirs(logpath, mode=0755)
                except OSError, e:
                    self.errorlog(0, _('Cannot make directory for logfile %s' % logpath))
                    rc = False
            try:
                logfd = os.open(self.conf.logfile, os.O_WRONLY |
                                os.O_APPEND | os.O_CREAT, 0644)
            except OSError, e:
                self.errorlog(0, _('Cannot open logfile %s' % self.conf.logfile))
                rc = False
    
            logfile =  os.fdopen(logfd, 'a')
            fcntl.fcntl(logfd, fcntl.F_SETFD)
            self.filelog = Logger(threshold = 10, file_object = logfile, 
                            preprefix = self.printtime)
        else:
            self.filelog = Logger(threshold = 10, file_object = None, 
                            preprefix = self.printtime)
        return rc   

    def printtime(self):
        """ Overloading printtime in YumOutput Class"""
        return time.strftime('%b %d %H:%M:%S (yumex) ', time.localtime(time.time()))

        
    def doGtkEvents( self ):
        while gtk.events_pending():      # process gtk events
            gtk.main_iteration()
          


class CacheProgressCallback:

     '''
     The class handles text output callbacks during metadata cache updates.
     '''
     
     def __init__( self, log, errorlog, filelog=None, console=None, progress=None ):
          self.log = log
          self.errorlog = errorlog
          self.filelog = filelog
          self.console = console
          self.progress = progress

     def log( self, level, message ):
          self.log( level, message )

     def errorlog( self, level, message ):
          if self.errorlog:
                self.errorlog( level, message )

     def filelog( self, level, message ):
          if self.filelog:
                self.filelog( level, message )

     def progressbar( self, current, total, name=None ):
        if current == 1:
            #self.console.write_line( "Loading metadata from : %s \n" % name )
            if self.progress != None:
                self.progress.set_extraLabel( _("Processing metadata from : %s") % name )
                self.progress.set_etaLabel( "" )
                
        else:
            if self.progress != None:
                if current % 10 == 0 or current == total:
                    if total != 0:
                         frac = float( current )/float( total )
                    else:
                         frac = 0.0                    
                    txt = "%i/%i" % ( current, total )
                    self.progress.set_progress( frac, txt )                       
                while gtk.events_pending():      # process gtk events
                    gtk.main_iteration()

class MyBaseMeter( BaseMeter ):
     """ GUI Version of BaseMeter Class """  
     def __init__( self ):
          BaseMeter.__init__( self )
            
     def update( self, amount_read, now=None ):
          while gtk.events_pending():      # process gtk events
                gtk.main_iteration()
          BaseMeter.update( self, amount_read, now ) 
          
class MyTextMeter( MyBaseMeter ):
     """ Customized version of TextMeter class """
     def __init__( self, console, progress=None ):
          MyBaseMeter.__init__( self )
          self.console = console 
          self.progress = progress
          self.totSize = ""

     def _do_start( self, now=None ):
        if self.progress != None:
            self.progress.show() 
            if self.text is not None:
                text = self.text
            else:
                text = self.basename
            self.progress.set_progress( 0, " " )
            self.progress.set_extraLabel( text )
            self.progress.set_etaLabel( "" )
            if not self.size is None:
                self.totSize = format_number( self.size )

     def _do_update( self, amount_read, now=None ):
          etime = self.re.elapsed_time()
          fetime = format_time( etime )
          fread = format_number( amount_read )
          #self.size = None
          if self.text is not None:
              text = self.text
          else:
              text = self.basename
          
          if self.size is None:
                #out = '%-60s : %5sB %s ' % ( text, fread, fetime )
                out = '%s' % ( text )
                eta = '%5sB %s' % (fread, fetime )
                
                frac = 0.0
          else:
                rtime = self.re.remaining_time()
                frtime = format_time( rtime )
                frac = self.re.fraction_read()
                out = '%s' % ( text )
                eta = '%5sB/%5sB %8s ETA' % ( fread, self.totSize, frtime )
                #out = '%-60s : %5sB/%5sB %8s ETA ' % ( text, fread, self.totSize, frtime )
          #self.console.write( out )
          if self.progress != None:
              percent  = "%3i%%" % int( frac*100 )
              self.progress.set_progress( frac, percent )
              self.progress.set_extraLabel( out )
              self.progress.set_etaLabel( eta )
              

     def _do_end( self, amount_read, now=None ):
          total_time = format_time( self.re.elapsed_time() )
          total_size = format_number( amount_read )
          if self.text is not None:
                text = self.text
          else:
                text = self.basename
          if self.progress != None:
              self.progress.set_progress( 1.0, "%3i%%" % 100 )
              self.progress.set_extraLabel( text )
              self.progress.set_etaLabel( "" )

class myRPMInstallCallback( callback.RPMInstallCallback ):

    def __init__( self, output=1, console=None, progress=None ):
        callback.RPMInstallCallback.__init__( self, output )
        self.console = console
        self.progress = progress

    def callback( self, what, bytes, total, h, user ):
          while gtk.events_pending():      # process gtk events
                gtk.main_iteration()      
          if what == rpm.RPMCALLBACK_TRANS_START:
                if bytes == 6:
                     self.total_actions = total

          elif what == rpm.RPMCALLBACK_TRANS_PROGRESS:
                pass

          elif what == rpm.RPMCALLBACK_TRANS_STOP:
                pass

          elif what == rpm.RPMCALLBACK_INST_OPEN_FILE:
                hdr = None
                if h is not None:
                     hdr, rpmloc = h
                     handle = self._makeHandle( hdr )
                     fd = os.open( rpmloc, os.O_RDONLY )
                     self.callbackfilehandles[handle]=fd
                     self.total_installed += 1
                     self.installed_pkg_names.append( hdr['name'] )
                     return fd
                else:
                     self._localprint( _( "No header - huh?" ) )

          elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
                hdr = None
                if h is not None:
                     hdr, rpmloc = h
                     handle = self._makeHandle( hdr )
                     os.close( self.callbackfilehandles[handle] )
                     fd = 0

                     # log stuff
                     pkgtup = self._dopkgtup( hdr )
                     txmbr = self.tsInfo.getMembers( pkgtup=pkgtup )[0] # if we have more than one I'll eat my hat
                     try:
                          process = self.myprocess[txmbr.output_state]
                          processed = self.mypostprocess[txmbr.output_state]
                     except KeyError, e:
                          pass

                     if self.filelog:
                          pkgrep = self._logPkgString( hdr )
                          msg = '%s: %s' % ( processed, pkgrep )
                          self.filelog( 0, msg )


          elif what == rpm.RPMCALLBACK_INST_PROGRESS:
                if h is not None:
                     hdr, rpmloc = h
                     if total == 0:
                          percent = 0
                     else:
                          percent = ( bytes*100L )/total
                     pkgtup = self._dopkgtup( hdr )
                     txmbr = self.tsInfo.getMembers( pkgtup=pkgtup )[0]
                     try:
                          process = self.myprocess[txmbr.output_state]
                     except KeyError, e:
                          print "Error: invalid output state: %s for %s" % \
                              ( txmbr.output_state, hdr['name'] )
                     else:
                          if self.output and sys.stdout.isatty():
                                fmt = self._makefmt( percent )
                                msg = fmt % ( process, hdr['name'] )
                                #self.console.write( msg )
                                if bytes == total:
                                     print " "
                          if self.progress != None:
                              out = "%s : %s" % ( process, hdr['name'] )
                              self.progress.set_extraLabel( out )
                              self.progress.set_etaLabel( "" )
                              self.progress.set_progress( float( percent )/100, "%3i%%" % percent )

          elif what == rpm.RPMCALLBACK_UNINST_START:
             if h not in self.installed_pkg_names:
                 process = "Removing"
             else:
                 process = "Cleanup"
             if self.progress != None:
                 out = "%s : %s" % ( process, h )
                 self.progress.set_extraLabel( out )
                 self.progress.set_etaLabel( "" )

          elif what == rpm.RPMCALLBACK_UNINST_PROGRESS:
                pass

          elif what == rpm.RPMCALLBACK_UNINST_STOP:
                self.total_removed += 1
#                if h not in self.installed_pkg_names:
#                     logmsg = _( 'Erased: %s' % ( h ) )
#                     if self.filelog: self.filelog( 0, logmsg )
                
                if self.output and sys.stdout.isatty():
                     percent = 100
                     #fmt = self._makefmt( percent, False )
                     #msg = fmt % ( process, h )
                     if self.progress != None:
                          self.progress.set_progress( float( percent )/100, "%3i%%" % percent )
                     

                     
class yumexAction:
    """ This class controls installing,updating,removing and yum setup"""    

    def __init__(self,xml,dlg,console,status):
        self.progressDlg = dlg
        self.status = status
        self.selected = None
        self.console = console
        self.yw = None
        self.xml = xml
        
    def set_yumbase(self,yw):
        self.yw = yw
        
    def set_selected(self,sel):
        self.selected = sel
        
    def set_repos(self,enabled,disabled):
        self.repoEnabled = enabled
        self.repoDisabled = disabled
                
    def install_packages( self ):
        self.set_status( _( "Preparing for install" ) )
        sel = self.selected
        state = True
        if len( sel ) > 0:
            self.progressDlg.show()
            self.progressDlg.set_mainLabel( _( "Installing Packages" ) )
            rc, msg = self.yw.install( sel)
            
            if rc != 2:
                info = _( "\nCheck output page for more info." )
                m = self.get_str(msg)
                rc = dialogs.warning_box( _( "Error" ), m+info )
                state = False
            else:
                if self.get_confimation( "Install" ):
                    if not self.process_packages( "Install" ):
                        state=False
                    else:
                        if 'autocleanup' in self.yw.options:
                            rc,msg = self.yw.cleanPackages()
                            self.yw.log(2,msg[0])
                else:
                    state = False
            self.progressDlg.hide()
            return state
        else:
            self.set_status( _( "No packages selected" ) )
            return state
            
        
    def remove_packages( self ):
        self.set_status( _( "Preparing for remove" ) )
        sel = self.selected
        state = True
        if len( sel ) > 0:
            self.progressDlg.show()
            self.progressDlg.set_mainLabel( _( "Removing Packages" ) )
            rc, msg = self.yw.remove( sel)
            if rc != 2:
                info = _( "\nCheck output page for more info." )
                m = self.get_str(msg)
                rc = dialogs.warning_box( _( "Error" ), m+info )
                state = False
            else:
                if self.get_confimation( "Remove" ):
                    if not self.process_packages( "Remove" ):
                        state=False
                else:
                    state = False
            self.progressDlg.hide()
            return state
        else:
            self.set_status( _( "No packages selected" ) )
            return state
        
    def update_packages( self ):
        sel = self.selected
        state = True
        if len( sel ) > 0:
            self.progressDlg.show()
            self.progressDlg.set_mainLabel( _( "Updating Packages" ) )
            self.set_status( _( "Preparing for update" ) )
            rc, msg = self.yw.update( sel)
            if rc != 2:
                info = _( "\nCheck output page for more info." )
                m = self.get_str(msg)
                rc = dialogs.warning_box( _( "Error" ), m+info )
                state=False
            else:
                if self.get_confimation( "Update" ):
                    if not self.process_packages( "Update" ):
                       state=False
                    else:
                        if 'autocleanup' in self.yw.options:
                            rc,msg = self.yw.cleanPackages()
                            self.yw.log(2,msg[0])
                else:
                    state=False
            self.progressDlg.hide()
            return state
        else:
            self.set_status( _( "No packages selected" ) )       
            return state
            
    def get_str(self,lst):
        print lst
        rc = ""
        for l in lst:
            rc = rc + l+"\n"
        return rc
            
    
    def set_status( self, text ):
        context_id = self.status.get_context_id( "Status" )
        self.status.push( context_id, text )
    
    def get_confimation( self, task ):
        self.yw.tsInfo.makelists()
        upd = self.yw.tsInfo.updated
        inst = self.yw.tsInfo.installed
        rmv = self.yw.tsInfo.removed
        ob = self.yw.tsInfo.obsoleted
        depup = self.yw.tsInfo.depupdated
        depinst = self.yw.tsInfo.depinstalled
        deprmv = self.yw.tsInfo.depremoved
        if task == "Install":
            pkg = self.get_package_list( inst )
            dep = self.get_package_list( depinst )
        elif task == "Update":
            if len( inst )>0:
                upd = upd + inst
            pkg = self.get_package_list( upd )
            dep = self.get_package_list( depup )
        elif task == "Remove":
            pkg = self.get_package_list( rmv )
            dep = self.get_package_list( deprmv )
        size = self.get_download_size()
        self.progressDlg.hide( False )
        rc = dialogs.confimation_dialog( self.xml, task, pkg, dep, size )
        self.progressDlg.show
        return rc

    def get_package_list( self, List ):
        txt = []
        if List == []: return ""
        for mbr in List:
            pkg_size = self.yw.format_number( float( mbr.po.size() ) )
            ver = mbr.po.printVer()
            el = ( mbr.name, ver,pkg_size )
            txt.append( el )
        return txt

    def get_download_size( self ):
        
        totsize = 0
        for txmbr in self.yw.tsInfo.getMembers():
            if txmbr.ts_state in ['i', 'u']:
                po = self.yw.getPackageObject( txmbr.pkgtup )
                if po:
                    size = int( po.size() )
                    totsize += size
        return self.yw.format_number( float( totsize ) )
        
    def debug_out( self, func, txt ):
        self.console.write_line( "DEBUG:(%s) - %s \n" % ( func, txt ), self.console.style_ps1 ) 
    

    def debug_showrepos( self ):
        enabled = ""
        for r in self.repoEnabled:
            enabled = enabled + "    "+r + '\n'
        self.console.write_line( _( "Enabled Repositories:\n%s" ) % enabled, self.console.style_ps1 ) 
             
                    
    def process_packages( self, task ):
        self.set_status( _( "%s packages (See Output for details)" ) % task )
        if not self.yw.process_transaction():            
        
            rc = dialogs.warning_box( _( "Error" ), _( "Error in Transaction" ) )
            return False
        else:
            rc = dialogs.warning_box( task, _( "%s completed ok" ) % task )
            return True
    
    
                     